@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2009-2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
@@ -18,10 +17,10 @@ Both derive from L{ControlledDict} which defines the C{changed} signal
 which can be used to track changes in the configuration.
 
 Typically these classes are not instantiated directly, but by the
-L{ConfigManager} defined in Lzim.config.manager}.
+L{ConfigManager} defined in L{zim.config.manager}.
 '''
 
-from __future__ import with_statement
+
 
 
 import sys
@@ -30,16 +29,10 @@ import logging
 import types
 import collections
 import ast
-
-
-if sys.version_info >= (2, 6):
-	import json # in standard lib since 2.6
-else: #pragma: no cover
-	import simplejson as json # extra dependency
-
+import json
 
 from zim.signals import SignalEmitter, ConnectorMixin, SIGNAL_NORMAL
-from zim.utils import OrderedDict, FunctionThread
+from zim.utils import OrderedDict
 from zim.fs import File, FileNotFoundError
 from zim.newfs import FileNotFoundError as NewFileNotFoundError
 from zim.errors import Error
@@ -50,12 +43,12 @@ from .basedirs import XDG_CONFIG_HOME
 logger = logging.getLogger('zim.config')
 
 
-class _MyMeta(SignalEmitter.__metaclass__, OrderedDict.__metaclass__):
+class _MyMeta(type(SignalEmitter), type(OrderedDict)):
 	# Combine meta classes to resolve conflict
 	pass
 
 
-class ControlledDict(OrderedDict, SignalEmitter, ConnectorMixin):
+class ControlledDict(OrderedDict, SignalEmitter, ConnectorMixin, metaclass=_MyMeta):
 	'''Sub-class of C{OrderedDict} that tracks modified state.
 	This modified state is recursive for nested C{ControlledDict}s.
 
@@ -65,8 +58,6 @@ class ControlledDict(OrderedDict, SignalEmitter, ConnectorMixin):
 	or a nested C{ControlledDict} changed
 	'''
 
-	__metaclass__ = _MyMeta
-
 	__signals__ = {
 		'changed': (SIGNAL_NORMAL, None, ())
 	}
@@ -120,7 +111,7 @@ class ControlledDict(OrderedDict, SignalEmitter, ConnectorMixin):
 			self._modified = True
 		else:
 			self._modified = False
-			for v in self.values():
+			for v in list(self.values()):
 				if isinstance(v, ControlledDict):
 					v.set_modified(False)
 
@@ -149,7 +140,7 @@ class ConfigDefinition(object):
 			if self.allow_empty:
 				return True
 			else:
-				raise ValueError, 'Value not allowed to be empty'
+				raise ValueError('Value not allowed to be empty')
 		else:
 			return False
 
@@ -190,14 +181,14 @@ class ConfigDefinitionByClass(ConfigDefinition):
 	'''
 	# TODO fully get rid of this class and replace by specialized classes
 
-	__slots__= ('klass',)
+	__slots__ = ('klass',)
 
 	def __init__(self, default, klass=None, allow_empty=False):
 		if klass is None:
 			klass = default.__class__
 
-		if issubclass(klass, basestring):
-			self.klass = basestring
+		if issubclass(klass, str):
+			self.klass = str
 		else:
 			self.klass = klass
 
@@ -210,8 +201,8 @@ class ConfigDefinitionByClass(ConfigDefinition):
 	def check(self, value):
 		if self._check_allow_empty(value):
 			return None
-		elif isinstance(value, basestring) \
-		and not self.klass is basestring:
+		elif isinstance(value, str) \
+		and not self.klass is str:
 			value = self._eval_string(value)
 
 		if isinstance(value, self.klass):
@@ -225,15 +216,15 @@ class ConfigDefinitionByClass(ConfigDefinition):
 				return self.klass.new_from_zim_config(value)
 			except:
 				logger.debug('Error while converting %s to %s', value, self.klass, exc_info=1)
-				raise ValueError, 'Can not convert %s to %s' % (value, self.klass)
+				raise ValueError('Can not convert %s to %s' % (value, self.klass))
 		else:
-			raise ValueError, 'Value should be of type: %s' % self.klass.__name__
+			raise ValueError('Value should be of type: %s' % self.klass.__name__)
 
 	def tostring(self, value):
 		if hasattr(value, 'serialize_zim_config'):
 			return value.serialize_zim_config()
 		else:
-			return json.dumps(value, separators=(',',':'))
+			return json.dumps(value, separators=(',', ':'))
 				# specify separators for compact encoding
 
 
@@ -248,7 +239,7 @@ class Boolean(ConfigDefinition):
 		elif value in ('True', 'true', 'False', 'false'):
 			return value in ('True', 'true')
 		else:
-			raise ValueError, 'Must be True or False'
+			raise ValueError('Must be True or False')
 
 
 class String(ConfigDefinition):
@@ -264,12 +255,12 @@ class String(ConfigDefinition):
 	def check(self, value):
 		if self._check_allow_empty(value):
 			return None
-		elif isinstance(value, basestring):
+		elif isinstance(value, str):
 			return value
 		elif hasattr(value, 'serialize_zim_config'):
 			return value.serialize_zim_config()
 		else:
-			raise ValueError, 'Must be string'
+			raise ValueError('Must be string')
 
 	def tostring(self, value):
 		if value is None:
@@ -299,7 +290,7 @@ class Integer(ConfigDefinition):
 			try:
 				return int(value)
 			except:
-				raise ValueError, 'Must be integer'
+				raise ValueError('Must be integer')
 
 
 class Float(ConfigDefinition):
@@ -314,7 +305,7 @@ class Float(ConfigDefinition):
 			try:
 				return float(value)
 			except:
-				raise ValueError, 'Must be integer'
+				raise ValueError('Must be integer')
 
 
 class Choice(ConfigDefinition):
@@ -342,8 +333,8 @@ class Choice(ConfigDefinition):
 			return None
 		else:
 			# Allow options that are not strings (e.g. tuples of strings)
-			if isinstance(value, basestring) \
-			and not all(isinstance(t, basestring) for t in self.choices):
+			if isinstance(value, str) \
+			and not all(isinstance(t, str) for t in self.choices):
 				value = self._eval_string(value)
 
 			# HACK to allow for preferences with "choice" item that has
@@ -360,10 +351,10 @@ class Choice(ConfigDefinition):
 
 			if value in choices:
 				return value
-			elif isinstance(value, basestring) and value.lower() in choices:
+			elif isinstance(value, str) and value.lower() in choices:
 				return value.lower()
 			else:
-				raise ValueError, 'Value should be one of %s' % unicode(choices)
+				raise ValueError('Value should be one of %s' % str(choices))
 
 
 class Range(Integer):
@@ -389,7 +380,7 @@ class Range(Integer):
 		elif self.min <= value <= self.max:
 			return value
 		else:
-			raise ValueError, 'Value should be between %i and %i' % (self.min, self.max)
+			raise ValueError('Value should be between %i and %i' % (self.min, self.max))
 
 
 class Coordinate(ConfigDefinition):
@@ -401,11 +392,11 @@ class Coordinate(ConfigDefinition):
 
 	def __init__(self, default, allow_empty=False):
 		if default == (None, None):
-			allow_empty=True
+			allow_empty = True
 		ConfigDefinition.__init__(self, default, allow_empty)
 
 	def check(self, value):
-		if isinstance(value, basestring):
+		if isinstance(value, str):
 			value = self._eval_string(value)
 
 		if self._check_allow_empty(value) \
@@ -423,15 +414,13 @@ class Coordinate(ConfigDefinition):
 			):
 				return value
 			else:
-				raise ValueError, 'Value should be a coordinate (tuple of 2 integers)'
+				raise ValueError('Value should be a coordinate (tuple of 2 integers)')
 
 value_is_coord = Coordinate # XXX for backward compatibility
 
 
 _definition_classes = {
 	str: String,
-	unicode: String,
-	basestring: String,
 	int: Integer,
 	float: Float,
 	bool: Boolean,
@@ -440,14 +429,14 @@ _definition_classes = {
 
 def build_config_definition(default=None, check=None, allow_empty=False):
 	'''Convenience method to construct a L{ConfigDefinition} object
-	based on a default value an/or a check.
+	based on a default value and/or a check.
 	'''
 	if default is None and check is None:
-		raise AssertionError, 'At least provide either a default or a check'
+		raise AssertionError('At least provide either a default or a check')
 	elif check is None:
 		check = default.__class__
 
-	if isinstance(check, (type, types.ClassType)): # is a class
+	if isinstance(check, type): # is a class
 		if issubclass(check, ConfigDefinition):
 			return check(default, allow_empty=allow_empty)
 		elif check in _definition_classes:
@@ -463,7 +452,7 @@ def build_config_definition(default=None, check=None, allow_empty=False):
 			and isinstance(check[1], int)
 		return Range(default, check[0], check[1])
 	else:
-		raise ValueError, 'Unrecognized check type'
+		raise ValueError('Unrecognized check type')
 
 
 
@@ -541,8 +530,8 @@ class ConfigDict(ControlledDict):
 		if k in self.definitions:
 			try:
 				v = self.definitions[k].check(v)
-			except ValueError, error:
-				raise ValueError, 'Invalid config value for %s: "%s" - %s' % (k, v, error.args[0])
+			except ValueError as error:
+				raise ValueError('Invalid config value for %s: "%s" - %s' % (k, v, error.args[0]))
 			else:
 				ControlledDict.__setitem__(self, k, v)
 		else:
@@ -567,7 +556,7 @@ class ConfigDict(ControlledDict):
 		assert not (E and F)
 		update = E or F
 		if hasattr(update, 'items'):
-			items = update.items()
+			items = list(update.items())
 		else:
 			items = update
 
@@ -587,16 +576,15 @@ class ConfigDict(ControlledDict):
 		assert not (E and F)
 		update = E or F
 		if isinstance(update, collections.Mapping):
-			items = update.items()
+			items = list(update.items())
 		else:
 			items = update
 
 		for key, definition in items:
 			if key in self.definitions:
 				if definition != self.definitions[key]:
-					raise AssertionError, \
-						'Key is already defined with different definition: %s\n%s != %s' \
-						% (key, definition, self.definitions[key])
+					raise AssertionError('Key is already defined with different definition: %s\n%s != %s'
+						% (key, definition, self.definitions[key]))
 				else:
 					continue
 
@@ -611,7 +599,7 @@ class ConfigDict(ControlledDict):
 	def _set_input(self, key, value):
 		try:
 			value = self.definitions[key].check(value)
-		except ValueError, error:
+		except ValueError as error:
 			logger.warn(
 				'Invalid config value for %s: "%s" - %s',
 					key, value, error.args[0]
@@ -783,7 +771,7 @@ class INIConfigFile(SectionedConfigDict):
 				# First emit top level to allow general changes
 				self.emit('changed')
 				with self.block_signals('changed'):
-					for section in self.values():
+					for section in list(self.values()):
 						section.emit('changed')
 				self.set_modified(False)
 
@@ -803,7 +791,7 @@ class INIConfigFile(SectionedConfigDict):
 		# Note that we explicitly do _not_ support comments on the end
 		# of a line. This is because "#" could be a valid character in
 		# a config value.
-		if isinstance(text, basestring):
+		if isinstance(text, str):
 			text = text.splitlines(True)
 
 		section = None
@@ -837,18 +825,6 @@ class INIConfigFile(SectionedConfigDict):
 		self.file.writelines(self.dump())
 		self.set_modified(False)
 
-	def write_async(self):
-		'''Write data asynchronously and set C{modified} to C{False}
-		@returns: an L{FunctionThread} object
-		'''
-		func = FunctionThread(
-			self.file,
-			self.file.writelines,
-			self.dump())
-		func.start()
-		self.set_modified(False)
-		return func
-
 	def dump(self):
 		'''Serialize the config to a "ini-style" config file.
 		@returns: a list of lines with text in "ini-style" formatting
@@ -867,7 +843,7 @@ class INIConfigFile(SectionedConfigDict):
 						logger.exception('Error serializing "%s" in section "[%s]"', key, name)
 			lines.append('\n')
 
-		for name, section in self.items():
+		for name, section in list(self.items()):
 			if not name.startswith('_'):
 				if isinstance(section, list):
 					for s in section:
@@ -908,7 +884,7 @@ class HierarchicDict(object):
 		self.dict['__defaults__'] = defaults or {}
 
 	def __getitem__(self, k):
-		if not isinstance(k, basestring):
+		if not isinstance(k, str):
 			k = k.name # assume zim path
 		return HierarchicDictFrame(self.dict, k)
 
