@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 
-# Copyright 2009,2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2009-2017 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''This module contains utilities to work with external applications
 it is based on the Freedesktop.org (XDG) Desktop Entry specification
@@ -10,10 +9,6 @@ The main class is the L{DesktopEntryFile} which maps the application
 definition in a specific desktop entry. Typically these are not
 constructed directly, but requested through the L{ApplicationManager}.
 
-Similar there is the L{CustomTool} class, which defines a custom tool
-defined within zim, and the L{CustomToolManager} to manage these
-definitions.
-
 Also there is the L{OpenWithMenu} which is the widget to render a menu
 with available applications for a specific file plus a dialog so the
 user can define a new command on the fly.
@@ -21,16 +16,16 @@ user can define a new command on the fly.
 
 import os
 import logging
-import gtk
-import gobject
+from gi.repository import Gtk
+from gi.repository import GObject
+from gi.repository import GdkPixbuf
 
 import zim.fs
 from zim.fs import File, Dir, TmpFile, cleanup_filename
-from zim.config import XDG_DATA_HOME, XDG_DATA_DIRS, XDG_CONFIG_HOME, \
-	data_dirs, SectionedConfigDict, INIConfigFile, json, ConfigManager
+from zim.config import XDG_DATA_HOME, XDG_DATA_DIRS, data_dirs, SectionedConfigDict, INIConfigFile
 from zim.parsing import split_quoted_strings, uri_scheme
 from zim.applications import Application, WebBrowser, StartFile
-from zim.gui.widgets import ui_environment, Dialog, ErrorDialog
+from zim.gui.widgets import Dialog, ErrorDialog, MessageDialog, strip_boolean_result
 
 
 logger = logging.getLogger('zim.gui.applications')
@@ -41,7 +36,7 @@ def _application_file(path, dirs):
 	# could also be stored as applications/kde/foo.desktop but not necessarily..
 	paths = [path]
 	if '-' in path:
-		for i in range(1, path.count('-')+1):
+		for i in range(1, path.count('-') + 1):
 			paths.append(path.replace('-', '/', i))
 
 	for dir in dirs:
@@ -118,8 +113,10 @@ def get_mimetype(obj):
 	@param obj: a L{File} object, or an URL
 	@returns: mimetype or C{None}
 	'''
-	if isinstance(obj, File):
-		return obj.get_mimetype()
+
+	for method in ('get_mimetype', 'mimetype'): # zim.fs.File, newfs
+		if hasattr(obj, method):
+			return getattr(obj, method)()
 	else:
 		scheme = uri_scheme(obj)
 		if scheme in (None, 'file'):
@@ -132,20 +129,20 @@ def get_mimetype(obj):
 
 
 try:
-	import gio
+	from gi.repository import Gio
 except ImportError:
-	gio = None
+	Gio = None
 
 _last_warning_missing_icon = None
 	# used to surpress redundant logging
 
 def get_mime_icon(file, size):
-	if not gio:
+	if not Gio:
 		return None
 
 	try:
-		f = gio.File(uri=file.uri)
-		info = f.query_info('standard::*')
+		f = Gio.File.new_for_uri(file.uri)
+		info = f.query_info('standard::*', Gio.FileQueryInfoFlags.NONE, None)
 		icon = info.get_icon()
 	except:
 		logger.exception('Failed to query info for file: %s', file)
@@ -153,9 +150,9 @@ def get_mime_icon(file, size):
 
 	global _last_warning_missing_icon
 
-	if isinstance(icon, gio.ThemedIcon):
+	if isinstance(icon, Gio.ThemedIcon):
 		names = icon.get_names()
-		icon_theme = gtk.icon_theme_get_default()
+		icon_theme = Gtk.IconTheme.get_default()
 		try:
 			icon_info = icon_theme.choose_icon(names, size, 0)
 			if icon_info:
@@ -165,7 +162,7 @@ def get_mime_icon(file, size):
 					logger.debug('Missing icons in icon theme: %s', names)
 					_last_warning_missing_icon = names
 				return None
-		except gobject.GError:
+		except GObject.GError:
 			logger.exception('Could not load icon for file: %s', file)
 			return None
 	else:
@@ -175,10 +172,10 @@ def get_mime_icon(file, size):
 def get_mime_description(mimetype):
 	# Check XML file /usr/share/mime/MEDIA/SUBTYPE.xml
 	# Find element "comment" with "xml:lang" attribute for the locale
-	from zim.config import XDG_DATA_DIRS
+	from zim.config import XDG_DATA_HOME, XDG_DATA_DIRS
 
 	media, subtype = mimetype.split('/', 1)
-	for dir in XDG_DATA_DIRS:
+	for dir in [XDG_DATA_HOME] + XDG_DATA_DIRS:
 		file = dir.file(('mime', media, subtype + '.xml'))
 		if file.exists():
 			return _read_comment_from(file)
@@ -195,18 +192,18 @@ def _read_comment_from(file):
 	xmlns = "{http://www.w3.org/XML/1998/namespace}"
 	xml = et.parse(file.path)
 	fallback = []
-	#~ print "FIND COMMENT", file, mylang
+	#~ print("FIND COMMENT", file, mylang)
 	for elt in xml.getroot():
 		if elt.tag.endswith('comment'):
-			lang = elt.attrib.get(xmlns+'lang', '')
+			lang = elt.attrib.get(xmlns + 'lang', '')
 			if lang == mylang:
 				return elt.text
-			elif not lang or mylang.startswith(lang+'_'):
+			elif not lang or mylang.startswith(lang + '_'):
 				fallback.append((lang, elt.text))
 			else:
 				pass
 	else:
-		#~ print "FALLBACK", fallback
+		#~ print("FALLBACK", fallback)
 		if fallback:
 			fallback.sort()
 			return fallback[-1][1] # longest match
@@ -299,7 +296,7 @@ class ApplicationManager(object):
 		## See comment in get_default_application()
 
 		if application is not None:
-			if not isinstance(application, basestring):
+			if not isinstance(application, str):
 				application = application.key
 
 			if not application.endswith('.desktop'):
@@ -430,6 +427,240 @@ class ApplicationManager(object):
 		return entries
 
 
+from zim.errors import Error
+from zim.parsing import is_win32_share_re, is_url_re, is_uri_re
+
+from zim.fs import adapt_from_newfs, normalize_win32_share
+from zim.newfs import FileNotFoundError
+
+from zim.gui.widgets import QuestionDialog
+
+
+class NoApplicationFoundError(Error):
+	'''Exception raised when an application was not found'''
+	pass
+
+
+def open_file(widget, file, mimetype=None, callback=None):
+	'''Open a file or folder
+
+	@param widget: parent for new dialogs, C{Gtk.Widget} or C{None}
+	@param file: a L{File} or L{Folder} object
+	@param mimetype: optionally specify the mimetype to force a
+	specific application to open this file
+	@param callback: callback function to be passed on to
+	L{Application.spawn()} (if the application supports a
+	callback, otherwise it is ignored silently)
+
+	@raises FileNotFoundError: if C{file} does not exist
+	@raises NoApplicationFoundError: if a specific mimetype was
+	given, but no default application is known for this mimetype
+	(will not use fallback in this case - fallback would
+	ignore the specified mimetype)
+	'''
+	logger.debug('open_file(%s, %s)', file, mimetype)
+	file = adapt_from_newfs(file)
+	assert isinstance(file, (File, Dir))
+	if isinstance(file, (File)) and file.isdir():
+		file = Dir(file.path)
+
+	if not file.exists():
+		raise FileNotFoundError(file)
+
+	if isinstance(file, File): # File
+		manager = ApplicationManager()
+		if mimetype is None:
+			entry = manager.get_default_application(file.get_mimetype())
+			if entry is not None:
+				_open_with(widget, entry, file, callback)
+			else:
+				_open_with_filebrowser(widget, file, callback)
+
+		else:
+			entry = manager.get_default_application(mimetype)
+			if entry is not None:
+				_open_with(widget, entry, file, callback)
+			else:
+				raise NoApplicationFoundError('No Application found for: %s' % mimetype)
+				# Do not go to fallback, we can not force
+				# mimetype for fallback
+	else: # Dir
+		_open_with_filebrowser(widget, file, callback)
+
+
+def open_folder_prompt_create(widget, folder):
+	'''Open a folder and prompts to create it if it doesn't exist yet.
+	@param widget: parent for new dialogs, C{Gtk.Widget} or C{None}
+	@param folder: a L{Folder} object
+	'''
+	try:
+		open_folder(widget, folder)
+	except FileNotFoundError:
+		if QuestionDialog(widget, (
+			_('Create folder?'),
+				# T: Heading in a question dialog for creating a folder
+			_('The folder "%s" does not yet exist.\nDo you want to create it now?') % folder.basename
+				# T: Text in a question dialog for creating a folder, %s will be the folder base name
+		)).run():
+			folder.touch()
+			open_folder(widget, folder)
+
+
+def open_folder(widget, folder):
+	'''Open a folder.
+	@param widget: parent for new dialogs, C{Gtk.Widget} or C{None}
+	@param folder: a L{Folder} object
+	@raises FileNotFoundError: if C{folder} does not exist
+	see L{open_folder_prompt_create} for alternative behavior when folder
+	does not exist.
+	'''
+	dir = adapt_from_newfs(folder)
+	open_file(widget, dir)
+
+
+def open_url(widget, url):
+	'''Open an URL (or URI) in the web browser or other relevant
+	program. The application is determined based on the URL / URI
+	scheme. Unkown schemes and "file://" URIs are opened with the
+	webbrowser.
+	@param widget: parent for new dialogs, C{Gtk.Widget} or C{None}
+	@param url: an URL or URI as string
+	'''
+	logger.debug('open_url(%s)', url)
+	assert isinstance(url, str)
+
+	if is_win32_share_re.match(url):
+		url = normalize_win32_share(url)
+		if os.name == 'nt':
+			return _open_with_filebrowser(widget, url)
+		# else consider as a x-scheme-handler/smb type URI below
+	elif not is_uri_re.match(url):
+		raise ValueError('Not an URL: %s' % url)
+	else:
+		pass
+
+	if url.startswith('file:/'):
+		# Special case, force to browser (and not use open_file())
+		# even though the result may be the same if the browser is
+		# dispatched through xdg-open, gnome-open, ...)
+		_open_with_webbrowser(widget, url)
+	elif url.startswith('outlook:') and hasattr(os, 'startfile'):
+		# Special case for outlook folder paths on windows
+		os.startfile(url)
+	else:
+		from zim.gui.applications import get_mimetype
+		manager = ApplicationManager()
+		type = get_mimetype(url) # Supports "x-scheme-handler/... for URL schemes"
+		logger.debug('Got type "%s" for "%s"', type, url)
+		entry = manager.get_default_application(type)
+		if entry:
+			_open_with(widget, entry, url)
+		elif url.startswith('mailto:'):
+			_open_with_emailclient(widget, url)
+		else:
+			_open_with_webbrowser(widget, url)
+
+
+def _open_with(widget, entry, uri, callback=None):
+	def check_error(status):
+		if status != 0:
+				ErrorDialog(widget, _('Could not open: %s') % uri).run()
+				# T: error when external application fails
+
+	if callback is None:
+		callback = check_error
+
+	try:
+		entry.spawn((uri,), callback=callback)
+	except NotImplementedError:
+		entry.spawn((uri,)) # E.g. webbrowser module does not support callback
+
+
+def _open_with_filebrowser(widget, file, callback=None):
+	entry = ApplicationManager.get_fallback_filebrowser()
+	_open_with(widget, entry, file, callback)
+
+
+def _open_with_emailclient(widget, uri):
+	entry = ApplicationManager.get_fallback_emailclient()
+	_open_with(widget, entry, uri)
+
+
+def _open_with_webbrowser(widget, url):
+	entry = ApplicationManager.get_fallback_webbrowser()
+	_open_with(widget, entry, url)
+
+
+def edit_config_file(widget, configfile):
+	'''Edit a config file in an external editor.
+	See L{edit_file()} for details.
+	@param widget: a C{gtk} widget to use as parent for dialogs or C{None}
+	@param configfile: a L{ConfigFile} object
+	'''
+	configfile.touch()
+	edit_file(widget, configfile.file, istextfile=True)
+
+
+def edit_file(widget, file, istextfile=None):
+	'''Edit a file with and external application.
+
+	This method will show a dialog to block the interface while the
+	external application is running. The dialog is closed
+	automatically when the application exits _after_ modifying the
+	file. If the file is unmodified the user needs to click the
+	"Done" button in the dialog because we can not know if the
+	application was really done or just forked to another process.
+
+	@param widget: a C{gtk} widget to use as parent for dialogs or C{None}
+	@param file: a L{File} object
+	@param istextfile: if C{True} the text editor is used, otherwise
+	we ask the file browser for the correct application. When
+	C{None} we check the mimetype of the file to determine if it
+	is text or not.
+	'''
+	## FIXME force using real text editor, even when file has not
+	## text mimetype. This now goes wrong when editing e.g. a html
+	## template when the editor is "xdg-open" on linux or default
+	## os.startfile() on windows...
+
+	if not file.exists():
+		raise FileNotFoundError(file)
+
+	oldmtime = file.mtime()
+
+	dialog = MessageDialog(widget, (
+		_('Editing file: %s') % file.basename,
+			# T: main text for dialog for editing external files
+		_('You are editing a file in an external application. You can close this dialog when you are done')
+			# T: description for dialog for editing external files
+	))
+
+	def check_close_dialog(status):
+		if status != 0:
+			dialog.destroy()
+			ErrorDialog(widget, _('Could not open: %s') % file.basename).run()
+				# T: error when external application fails
+		else:
+			newmtime = file.mtime()
+			if newmtime != oldmtime:
+				dialog.destroy()
+
+	if istextfile:
+		try:
+			open_file(widget, file, mimetype='text/plain', callback=check_close_dialog)
+		except NoApplicationFoundError:
+			app = AddApplicationDialog(widget, 'text/plain').run()
+			if app:
+				# Try again
+				open_file(widget, file, mimetype='text/plain', callback=check_close_dialog)
+			else:
+				return # Dialog was cancelled, no default set, ...
+	else:
+		open_file(widget, file, callback=check_close_dialog)
+
+	dialog.run()
+
+
 from zim.config import String as BaseString
 from zim.config import Boolean as BaseBoolean
 from zim.config import Float as Numeric
@@ -439,9 +670,13 @@ class String(BaseString):
 	def check(self, value):
 		# Only ascii chars allowed in these keys
 		value = BaseString.check(self, value)
-		if isinstance(value, unicode) \
-		and value.encode('utf-8') != value:
-			raise ValueError, 'ASCII string required'
+		if isinstance(value, str):
+			try:
+				x = value.encode('ascii')
+			except UnicodeEncodeError:
+				raise ValueError('ASCII string required')
+			else:
+				pass
 		return value
 
 
@@ -449,6 +684,15 @@ class LocaleString(BaseString):
 	pass # utf8 already supported by default
 
 
+class IconString(LocaleString):
+
+	def check(self, value):
+		if hasattr(value, 'path'):
+			return value.path  # prevent fallback via serialize_zim_config to user_path
+		else:
+			return LocaleString.check(self, value)
+
+
 class Boolean(BaseBoolean):
 
 	def tostring(self, value):
@@ -490,25 +734,22 @@ class DesktopEntryDict(SectionedConfigDict, Application):
 	_definitions = (
 		# Data types for all keys are defined in spec - see freedesktop.org
 		# Don't define all keys in the spec, just define the ones that we might use
-		('Type',		String('Application')),
-		('Version',		Numeric(1.0)),
-		('GenericName',	LocaleString(None)),
-		('Name',		LocaleString(None)),
-		('Comment',		LocaleString(None)),
-		('Exec',		String(None)),
-		('TryExec',		String(None)),
-		('Icon',		LocaleString(None)),
-		('MimeType',	String(None)),
-		('Terminal',	Boolean(False)),
-		('NoDisplay',	Boolean(False)),
+		('Type', String('Application')),
+		('Version', Numeric(1.0)),
+		('GenericName', LocaleString(None)),
+		('Name', LocaleString(None)),
+		('Comment', LocaleString(None)),
+		('Exec', String(None)),
+		('TryExec', String(None)),
+		('Icon', IconString(None)),
+		('MimeType', String(None)),
+		('Terminal', Boolean(False)),
+		('NoDisplay', Boolean(False)),
 	)
 
 	def __init__(self):
 		SectionedConfigDict.__init__(self)
 		self['Desktop Entry'].define(self._definitions)
-		self.encoding = zim.fs.ENCODING
-		if self.encoding == 'mbcs':
-			self.encoding = 'utf-8'
 
 	@property
 	def key(self):
@@ -516,7 +757,7 @@ class DesktopEntryDict(SectionedConfigDict, Application):
 
 	def isvalid(self):
 		'''Check if all the fields that are required according to the
-		spcification are set. Assumes we only use desktop files to
+		specification are set. Assumes we only use desktop files to
 		describe applications (and not links or dirs, which are also
 		covered by the spec).
 		@returns: C{True} if all required fields are set
@@ -554,7 +795,7 @@ class DesktopEntryDict(SectionedConfigDict, Application):
 		return split_quoted_strings(self['Desktop Entry']['Exec'])
 
 	def get_pixbuf(self, size):
-		'''Get the application icon as a C{gtk.gdk.Pixbuf}.
+		'''Get the application icon as a C{GdkPixbuf.Pixbuf}.
 		@param size: the icon size as gtk constant
 		@returns: a pixbuf object or C{None}
 		'''
@@ -565,18 +806,18 @@ class DesktopEntryDict(SectionedConfigDict, Application):
 		if isinstance(icon, File):
 			icon = icon.path
 
-		w, h = gtk.icon_size_lookup(size)
+		w, h = strip_boolean_result(Gtk.icon_size_lookup(size))
 
 		if '/' in icon or '\\' in icon:
-			if zim.fs.isfile(icon):
-				return gtk.gdk.pixbuf_new_from_file_at_size(icon, w, h)
+			if os.path.isfile(icon):
+				return GdkPixbuf.Pixbuf.new_from_file_at_size(icon, w, h)
 			else:
 				return None
 		else:
-			theme = gtk.icon_theme_get_default()
+			theme = Gtk.IconTheme.get_default()
 			try:
-				pixbuf = theme.load_icon(icon.encode('utf-8'), w, 0)
-			except Exception, error:
+				pixbuf = theme.load_icon(icon, w, 0)
+			except Exception as error:
 				#~ logger.exception('Foo')
 				return None
 			return pixbuf
@@ -595,23 +836,27 @@ class DesktopEntryDict(SectionedConfigDict, Application):
 				if isinstance(arg, (File, Dir)):
 					uris.append(arg.uri)
 				else:
-					uris.append(unicode(arg))
+					uris.append(str(arg))
 			return uris
 
 		cmd = split_quoted_strings(self['Desktop Entry']['Exec'])
 		if args is None or len(args) == 0:
-			if '%f' in cmd: cmd.remove('%f')
-			elif '%F' in cmd: cmd.remove('%F')
-			elif '%u' in cmd: cmd.remove('%u')
-			elif '%U' in cmd: cmd.remove('%U')
+			if '%f' in cmd:
+				cmd.remove('%f')
+			elif '%F' in cmd:
+				cmd.remove('%F')
+			elif '%u' in cmd:
+				cmd.remove('%u')
+			elif '%U' in cmd:
+				cmd.remove('%U')
 		elif '%f' in cmd:
 			assert len(args) == 1, 'application takes one file name'
 			i = cmd.index('%f')
-			cmd[i] = unicode(args[0])
+			cmd[i] = str(args[0])
 		elif '%F' in cmd:
 			i = cmd.index('%F')
-			for arg in reversed(map(unicode, args)):
-				cmd.insert(i, unicode(arg))
+			for arg in reversed(list(map(str, args))):
+				cmd.insert(i, str(arg))
 			cmd.remove('%F')
 		elif '%u' in cmd:
 			assert len(args) == 1, 'application takes one url'
@@ -620,10 +865,10 @@ class DesktopEntryDict(SectionedConfigDict, Application):
 		elif '%U' in cmd:
 			i = cmd.index('%U')
 			for arg in reversed(uris(args)):
-				cmd.insert(i, unicode(arg))
+				cmd.insert(i, str(arg))
 			cmd.remove('%U')
 		else:
-			cmd.extend(map(unicode, args))
+			cmd.extend(list(map(str, args)))
 
 		if '%i' in cmd:
 			if 'Icon' in self['Desktop Entry'] \
@@ -668,8 +913,8 @@ class DesktopEntryFile(DesktopEntryDict, INIConfigFile):
 		return self.file.basename[:-8] # len('.desktop') is 8
 
 
-class OpenWithMenu(gtk.Menu):
-	'''Sub-class of C{gtk.Menu} implementing an "Open With..." menu with
+class OpenWithMenu(Gtk.Menu):
+	'''Sub-class of C{Gtk.Menu} implementing an "Open With..." menu with
 	applications to open a specific file. Also has an item
 	"Customize...", which opens a L{CustomizeOpenWithDialog}
 	and allows the user to add custom commands.
@@ -687,7 +932,7 @@ class OpenWithMenu(gtk.Menu):
 		known. Providing this arguments prevents redundant lookups of
 		the type (which is slow).
 		'''
-		gtk.Menu.__init__(self)
+		GObject.GObject.__init__(self)
 		self._window = widget.get_toplevel()
 		self.file = file
 		if mimetype is None:
@@ -701,14 +946,14 @@ class OpenWithMenu(gtk.Menu):
 			item.connect('activate', self.on_activate)
 
 		if not self.get_children():
-			item = gtk.MenuItem(_('No Applications Found'))
+			item = Gtk.MenuItem.new_with_mnemonic(_('No Applications Found'))
 				# T: message when no applications in "Open With" menu
 			item.set_sensitive(False)
 			self.append(item)
 
-		self.append(gtk.SeparatorMenuItem())
+		self.append(Gtk.SeparatorMenuItem())
 
-		item = gtk.MenuItem(self.CUSTOMIZE)
+		item = Gtk.MenuItem.new_with_mnemonic(self.CUSTOMIZE)
 		item.connect('activate', self.on_activate_customize, mimetype)
 		self.append(item)
 
@@ -720,7 +965,7 @@ class OpenWithMenu(gtk.Menu):
 		CustomizeOpenWithDialog(self._window, mimetype=mimetype).run()
 
 
-class DesktopEntryMenuItem(gtk.ImageMenuItem):
+class DesktopEntryMenuItem(Gtk.MenuItem):
 	'''Single menu item for the L{OpenWithMenu}. Displays the application
 	name and the icon.
 	'''
@@ -730,16 +975,11 @@ class DesktopEntryMenuItem(gtk.ImageMenuItem):
 
 		@param entry: the L{DesktopEntryFile}
 		'''
-		text = _('Open with "%s"') % entry.name
+		GObject.GObject.__init__(self)
+		self.set_label(_('Open with "%s"') % entry.name)
 			# T: menu item to open a file with an application, %s is the app name
-		gtk.ImageMenuItem.__init__(self, text)
 		self.entry = entry
 
-		if hasattr(entry, 'get_pixbuf'):
-			pixbuf = entry.get_pixbuf(gtk.ICON_SIZE_MENU)
-			if pixbuf:
-				self.set_image(gtk.image_new_from_pixbuf(pixbuf))
-
 
 def _mimetype_dialog_text(mimetype):
 		if mimetype.startswith('x-scheme-handler/'):
@@ -754,27 +994,27 @@ def _mimetype_dialog_text(mimetype):
 
 class CustomizeOpenWithDialog(Dialog):
 
-	def __init__(self, ui, mimetype):
+	def __init__(self, parent, mimetype):
 		'''Constructor
-		@param ui: the parent window or C{GtkInterface} object
+		@param parent: the parent window or C{None}
 		@param mimetype: mime-type for which we want to create a new
 		application
 		'''
-		Dialog.__init__(self, ui, _('Configure Applications'),  # T: Dialog title
-			buttons=gtk.BUTTONS_CLOSE, help='Help:Default Applications')
+		Dialog.__init__(self, parent, _('Configure Applications'),  # T: Dialog title
+			buttons=Gtk.ButtonsType.CLOSE, help='Help:Default Applications')
 		self.mimetype = mimetype
 		self.add_text(_mimetype_dialog_text(mimetype))
 
 		# Combo to set default
 		self.default_combo = ApplicationComboBox()
 		self.default_combo.connect('changed', self.on_default_changed)
-		hbox = gtk.HBox(spacing=12)
+		hbox = Gtk.HBox(spacing=12)
 		self.vbox.add(hbox)
-		hbox.pack_start(gtk.Label(_('Default')+':'), False) # T: label for default application
-		hbox.pack_start(self.default_combo)
+		hbox.pack_start(Gtk.Label(_('Default') + ':'), False, True, 0) # T: label for default application
+		hbox.pack_start(self.default_combo, True, True, 0)
 
 		# Button to add new
-		button = gtk.Button(_('Add Application'))
+		button = Gtk.Button.new_with_mnemonic(_('Add Application'))
 			# T: Button for adding a new application to the 'open with' menu
 		button.connect('clicked', self.on_add_application)
 		self.add_extra_button(button)
@@ -826,22 +1066,24 @@ class SystemDefault(object):
 
 
 
-class ApplicationComboBox(gtk.ComboBox):
+class ApplicationComboBox(Gtk.ComboBox):
 
 	NAME_COL = 0
 	APP_COL = 1
 	ICON_COL = 2
 
 	def __init__(self):
-		model = gtk.ListStore(str, object, gtk.gdk.Pixbuf) # NAME_COL, APP_COL, ICON_COL
-		gtk.ComboBox.__init__(self, model)
+		GObject.GObject.__init__(self)
+		self.set_model(
+			Gtk.ListStore(str, object, GdkPixbuf.Pixbuf) # NAME_COL, APP_COL, ICON_COL
+		)
 
-		cell = gtk.CellRendererPixbuf()
+		cell = Gtk.CellRendererPixbuf()
 		self.pack_start(cell, False)
 		cell.set_property('xpad', 5)
 		self.add_attribute(cell, 'pixbuf', self.ICON_COL)
 
-		cell = gtk.CellRendererText()
+		cell = Gtk.CellRendererText()
 		self.pack_start(cell, True)
 		cell.set_property('xalign', 0.0)
 		cell.set_property('xpad', 5)
@@ -853,9 +1095,9 @@ class ApplicationComboBox(gtk.ComboBox):
 	def append(self, application):
 		model = self.get_model()
 		if hasattr(application, 'get_pixbuf'):
-			pixbuf = application.get_pixbuf(gtk.ICON_SIZE_MENU)
+			pixbuf = application.get_pixbuf(Gtk.IconSize.MENU)
 		else:
-			pixbuf = self.render_icon(gtk.STOCK_EXECUTE, gtk.ICON_SIZE_MENU)
+			pixbuf = self.render_icon(Gtk.STOCK_EXECUTE, Gtk.IconSize.MENU)
 
 		model.append((application.name, application, pixbuf)) # NAME_COL, APP_COL, ICON_COL
 
@@ -874,20 +1116,20 @@ class AddApplicationDialog(Dialog):
 	L{ApplicationManager.create()}.
 	'''
 
-	def __init__(self, ui, mimetype):
+	def __init__(self, parent, mimetype):
 		'''Constructor
-		@param ui: the parent window or C{GtkInterface} object
+		@param parent: the parent window or C{None}
 		@param mimetype: mime-type for which we want to create a new
 		application
 		'''
-		Dialog.__init__(self, ui, _('Add Application')) # T: Dialog title
+		Dialog.__init__(self, parent, _('Add Application')) # T: Dialog title
 		self.mimetype = mimetype
 		self.add_text(_mimetype_dialog_text(mimetype))
-		self.add_form( (
+		self.add_form((
 			('name', 'string', _('Name')), # T: Field in 'custom command' dialog
 			('exec', 'string', _('Command')), # T: Field in 'custom command' dialog
 			('default', 'bool', _('Make default application')), # T: Field in 'custom command' dialog
-		) )
+		))
 		self.form['default'] = True
 
 	def do_response_ok(self):
@@ -913,292 +1155,3 @@ class AddApplicationDialog(Dialog):
 
 		self.result = application
 		return True
-
-
-class CustomToolManager(object):
-	'''Manager for dealing with the desktop files which are used to
-	store custom tools.
-
-	Custom tools are external commands that are intended to show in the
-	"Tools" menu in zim (and optionally in the tool bar). They are
-	defined as desktop entry files in a special folder (typically
-	"~/.local/share/zim/customtools") and use several non standard keys.
-	See L{CustomTool} for details.
-
-	This object is iterable and maintains a specific order for tools
-	to be shown in in the user interface.
-	'''
-
-	def __init__(self):
-		self.config = ConfigManager() # XXX should be passed in
-		self.names = []
-		self.tools = {}
-		self._read_list()
-
-	def _read_list(self):
-		file = self.config.get_config_file('customtools/customtools.list')
-		seen = set()
-		for line in file.readlines():
-			name = line.strip()
-			if not name in seen:
-				seen.add(name)
-				self.names.append(name)
-
-	def _write_list(self):
-		file = self.config.get_config_file('customtools/customtools.list')
-		file.writelines([name + '\n' for name in self.names])
-
-	def __iter__(self):
-		for name in self.names:
-			tool = self.get_tool(name)
-			if tool and tool.isvalid():
-				yield tool
-
-	def get_tool(self, name):
-		'''Get a L{CustomTool} by name.
-		@param name: the tool name
-		@returns: a L{CustomTool} object
-		'''
-		if not name in self.tools:
-			file = self.config.get_config_file('customtools/%s.desktop' % name)
-			tool = CustomTool(file)
-			self.tools[name] = tool
-
-		return self.tools[name]
-
-	def create(self, Name, **properties):
-		'''Create a new custom tool
-
-		@param Name: the name to show in the Tools menu
-		@param properties: properties for the custom tool, e.g.:
-		  - Comment
-		  - Icon
-		  - X-Zim-ExecTool
-		  - X-Zim-ReadOnly
-		  - X-Zim-ShowInToolBar
-
-		@returns: a new L{CustomTool} object.
-		'''
-		properties['Type'] = 'X-Zim-CustomTool'
-		dir = XDG_CONFIG_HOME.subdir('zim/customtools')
-		tool = _create_application(dir, Name, '', klass=CustomTool, NoDisplay=False, **properties)
-		self.tools[tool.key] = tool
-		self.names.append(tool.key)
-		self._write_list()
-
-		return tool
-
-	def delete(self, tool):
-		'''Remove a custom tool from the list and delete the definition
-		file.
-		@param tool: a custom tool name or L{CustomTool} object
-		'''
-		if not isinstance(tool, CustomTool):
-			tool = self.get_tool(tool)
-		tool.file.remove()
-		self.tools.pop(tool.key)
-		self.names.remove(tool.key)
-		self._write_list()
-
-	def index(self, tool):
-		'''Get the position of a specific tool in the list.
-		@param tool: a custom tool name or L{CustomTool} object
-		@returns: an integer for the position
-		'''
-		if isinstance(tool, CustomTool):
-			tool = tool.key
-		return self.names.index(tool)
-
-	def reorder(self, tool, i):
-		'''Change the position of a tool in the list.
-		@param tool: a custom tool name or L{CustomTool} object
-		@param i: the new position as integer
-		'''
-		if not 0 <= i < len(self.names):
-			return
-
-		if isinstance(tool, CustomTool):
-			tool = tool.key
-
-		j = self.names.index(tool)
-		self.names.pop(j)
-		self.names.insert(i, tool)
-		# Insert before i. If i was before old position indeed before
-		# old item at that position. However if i was after old position
-		# if shifted due to the pop(), now it inserts after the old item.
-		# This is intended behavior to make all moves possible.
-		self._write_list()
-
-
-
-from zim.config import Choice
-
-class CustomToolDict(DesktopEntryDict):
-	'''This is a specialized desktop entry type that is used for
-	custom tools for the "Tools" menu in zim. It uses a non-standard
-	Exec spec with zim specific escapes for "X-Zim-ExecTool".
-
-	The following fields are expanded:
-		- C{%f} for source file as tmp file current page
-		- C{%d} for attachment directory
-		- C{%s} for real source file (if any)
-		- C{%n} for notebook location (file or directory)
-		- C{%D} for document root
-		- C{%t} for selected text or word under cursor
-		- C{%T} for the selected text including wiki formatting
-
-	Other additional keys are:
-		- C{X-Zim-ReadOnly} - boolean
-		- C{X-Zim-ShowInToolBar} - boolean
-		- C{X-Zim-ShowInContextMenu} - 'None', 'Text' or 'Page'
-
-	These tools should always be executed with 3 arguments: notebook,
-	page & pageview.
-	'''
-
-	_definitions = DesktopEntryDict._definitions + (
-			('X-Zim-ExecTool',			String(None)),
-			('X-Zim-ReadOnly',			Boolean(True)),
-			('X-Zim-ShowInToolBar',		Boolean(False)),
-			('X-Zim-ShowInContextMenu',	Choice(None, ('Text', 'Page'))),
-			('X-Zim-ReplaceSelection',	Boolean(False)),
-	)
-
-	def isvalid(self):
-		'''Check if all required fields are set.
-		@returns: C{True} if all required fields are set
-		'''
-		entry = self['Desktop Entry']
-		if entry.get('Type') == 'X-Zim-CustomTool' \
-		and entry.get('Version') == 1.0 \
-		and entry.get('Name') \
-		and entry.get('X-Zim-ExecTool') \
-		and not entry.get('X-Zim-ReadOnly') is None \
-		and not entry.get('X-Zim-ShowInToolBar') is None \
-		and 'X-Zim-ShowInContextMenu' in entry:
-			return True
-		else:
-			logger.error('Invalid custom tool entry: %s %s', self.key, entry)
-			return False
-
-	def get_pixbuf(self, size):
-		pixbuf = DesktopEntryDict.get_pixbuf(self, size)
-		if pixbuf is None:
-			pixbuf = gtk.Label().render_icon(gtk.STOCK_EXECUTE, size)
-			# FIXME hack to use arbitrary widget to render icon
-		return pixbuf
-
-	@property
-	def icon(self):
-		return self['Desktop Entry'].get('Icon') or gtk.STOCK_EXECUTE
-			# get('Icon', gtk.STOCK_EXECUTE) still returns empty string if key exists but no value
-
-	@property
-	def execcmd(self):
-		return self['Desktop Entry']['X-Zim-ExecTool']
-
-	@property
-	def isreadonly(self):
-		return self['Desktop Entry']['X-Zim-ReadOnly']
-
-	@property
-	def showintoolbar(self):
-		return self['Desktop Entry']['X-Zim-ShowInToolBar']
-
-	@property
-	def showincontextmenu(self):
-		return self['Desktop Entry']['X-Zim-ShowInContextMenu']
-
-	@property
-	def replaceselection(self):
-		return self['Desktop Entry']['X-Zim-ReplaceSelection']
-
-	def parse_exec(self, args=None):
-		if not (isinstance(args, tuple) and len(args) == 3):
-			raise AssertionError, 'Custom commands needs 3 arguments'
-			# assert statement could be optimized away
-		notebook, page, pageview = args
-
-		cmd = split_quoted_strings(self['Desktop Entry']['X-Zim-ExecTool'])
-		if '%f' in cmd:
-			self._tmpfile = TmpFile('tmp-page-source.txt')
-			self._tmpfile.writelines(page.dump('wiki'))
-			cmd[cmd.index('%f')] = self._tmpfile.path
-
-		if '%d' in cmd:
-			dir = notebook.get_attachments_dir(page)
-			if dir:
-				cmd[cmd.index('%d')] = dir.path
-			else:
-				cmd[cmd.index('%d')] = ''
-
-		if '%s' in cmd:
-			if hasattr(page, 'source') and isinstance(page.source, File):
-				cmd[cmd.index('%s')] = page.source.path
-			else:
-				cmd[cmd.index('%s')] = ''
-
-		if '%p' in cmd:
-			cmd[cmd.index('%p')] = page.name
-
-		if '%n' in cmd:
-			cmd[cmd.index('%n')] = File(notebook.uri).path
-
-		if '%D' in cmd:
-			dir = notebook.document_root
-			if dir:
-				cmd[cmd.index('%D')] = dir.path
-			else:
-				cmd[cmd.index('%D')] = ''
-
-		if '%t' in cmd:
-			text = pageview.get_selection() or pageview.get_word()
-			cmd[cmd.index('%t')] = text or ''
-			# FIXME - need to substitute this in arguments + url encoding
-
-		if '%T' in cmd:
-			text = pageview.get_selection(format='wiki') or pageview.get_word(format='wiki')
-			cmd[cmd.index('%T')] = text or ''
-			# FIXME - need to substitute this in arguments + url encoding
-
-		return tuple(cmd)
-
-	_cmd = parse_exec # To hook into Application.spawn and Application.run
-
-	def run(self, args, cwd=None):
-		self._tmpfile = None
-		Application.run(self, args, cwd=cwd)
-		if self._tmpfile:
-			notebook, page, pageview = args
-			page.parse('wiki', self._tmpfile.readlines())
-			notebook.store_page(page)
-			self._tmpfile = None
-
-	def update(self, E=(), **F):
-		self['Desktop Entry'].update(E, **F)
-
-		# Set sane default for X-Zim-ShowInContextMenus
-		if not (E and 'X-Zim-ShowInContextMenu' in E) \
-		and not 'X-Zim-ShowInContextMenu' in F:
-			cmd = split_quoted_strings(self['Desktop Entry']['X-Zim-ExecTool'])
-			if any(c in cmd for c in ['%f', '%d', '%s']):
-				context = 'Page'
-			elif '%t' in cmd:
-				context = 'Text'
-			else:
-				context = None
-			self['Desktop Entry']['X-Zim-ShowInContextMenu'] = context
-
-
-class CustomTool(CustomToolDict, INIConfigFile):
-	'''Class representing a file defining a custom tool, see
-	L{CustomToolDict} for the API documentation.
-	'''
-
-	def __init__(self, file):
-		CustomToolDict.__init__(self)
-		INIConfigFile.__init__(self, file)
-
-	@property
-	def key(self):
-		return self.file.basename[:-8] # len('.desktop') is 8
