@@ -1,6 +1,5 @@
-# -*- coding: utf-8 -*-
 
-# Copyright 2009 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2009-2018 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 '''Utilities to work with the clipboard for copy-pasting
 
@@ -9,7 +8,10 @@ functionality, which works similar to the clipboard, but has a less
 straight forward API.
 '''
 
-import gtk
+from gi.repository import Gtk
+from gi.repository import Gdk
+from gi.repository import GdkPixbuf
+
 import logging
 
 from zim.fs import File, Dir, FS
@@ -33,22 +35,35 @@ PARSETREE_TARGET = (PARSETREE_TARGET_NAME, 0, PARSETREE_TARGET_ID)
 INTERNAL_PAGELIST_TARGET_ID = 2
 INTERNAL_PAGELIST_TARGET_NAME = 'text/x-zim-page-list-internal'
 INTERNAL_PAGELIST_TARGET = \
-	(INTERNAL_PAGELIST_TARGET_NAME, gtk.TARGET_SAME_APP, INTERNAL_PAGELIST_TARGET_ID)
+	(INTERNAL_PAGELIST_TARGET_NAME, Gtk.TargetFlags.SAME_APP, INTERNAL_PAGELIST_TARGET_ID)
 
 PAGELIST_TARGET_ID = 3
 PAGELIST_TARGET_NAME = 'text/x-zim-page-list'
 PAGELIST_TARGET = (PAGELIST_TARGET_NAME, 0, PAGELIST_TARGET_ID)
 
 IMAGE_TARGET_ID = 6
-IMAGE_TARGETS = tuple(gtk.target_list_add_image_targets(info=IMAGE_TARGET_ID))
-	# According to docs we should provide list as arg to this function,
-	# but seems docs are not correct
+IMAGE_TARGETS = tuple(
+	(name, 0, IMAGE_TARGET_ID) for name in [
+		'image/png', 'image/x-wmf', 'application/x-navi-animation',
+		'image/bmp', 'image/x-bmp', 'image/x-MS-bmp', 'image/gif',
+		'image/x-icns', 'image/x-icon', 'image/x-ico', 'image/x-win-bitmap',
+		'image/vnd.microsoft.icon', 'application/ico', 'image/ico',
+		'image/icon', 'text/ico', 'image/jpeg', 'image/x-portable-anymap',
+		'image/x-portable-bitmap', 'image/x-portable-graymap',
+		'image/x-portable-pixmap', 'image/x-quicktime', 'image/qtif',
+		'image/svg+xml', 'image/svg', 'image/svg-xml',
+		'image/vnd.adobe.svg+xml', 'text/xml-svg', 'image/svg+xml-compressed',
+		'image/x-tga', 'image/tiff', 'image/x-xbitmap', 'image/x-xpixmap'
+	] # TODO - check any win32 specific types to be added
+)
+#IMAGE_TARGETS = tuple(Gtk.target_list_add_image_targets(info=IMAGE_TARGET_ID))
 IMAGE_TARGET_NAMES = tuple([target[0] for target in IMAGE_TARGETS])
 
 # Add image format names as well, seen these being used by MS Office
-for format in gtk.gdk.pixbuf_get_formats():
-	if format['mime_types'][0] in IMAGE_TARGET_NAMES:
-		for n in (format['name'], format['name'].upper()):
+for format in GdkPixbuf.Pixbuf.get_formats():
+	if format.get_mime_types()[0] in IMAGE_TARGET_NAMES:
+		name = format.get_name()
+		for n in (name, name.upper()):
 			IMAGE_TARGET_NAMES += (n,)
 			IMAGE_TARGETS += ((n, 0, IMAGE_TARGET_ID),)
 
@@ -56,7 +71,8 @@ for format in gtk.gdk.pixbuf_get_formats():
 #~ print IMAGE_TARGET_NAMES
 
 URI_TARGET_ID = 7
-URI_TARGETS = tuple(gtk.target_list_add_uri_targets(info=URI_TARGET_ID))
+URI_TARGETS = (('text/uri-list', 0, URI_TARGET_ID),)
+#URI_TARGETS = tuple(Gtk.target_list_add_uri_targets(info=URI_TARGET_ID))
 	# According to docs we should provide list as arg to this function,
 	# but seems docs are not correct
 URI_TARGET_NAMES = tuple([target[0] for target in URI_TARGETS])
@@ -67,7 +83,13 @@ HTML_TARGET_NAMES = ('text/html', 'HTML Format')
 HTML_TARGETS = tuple([(name, 0, HTML_TARGET_ID) for name in HTML_TARGET_NAMES])
 
 TEXT_TARGET_ID = 9
-TEXT_TARGETS = tuple(gtk.target_list_add_text_targets(info=TEXT_TARGET_ID))
+TEXT_TARGETS = tuple(
+	(name, 0, TEXT_TARGET_ID) for name in [
+		'UTF8_STRING', 'COMPOUND_TEXT', 'TEXT', 'STRING',
+		'text/plain;charset=utf-8', 'text/plain'
+	]
+)
+#TEXT_TARGETS = tuple(Gtk.target_list_add_text_targets(info=TEXT_TARGET_ID))
 	# According to docs we should provide list as arg to this function,
 	# but seems docs are not correct
 TEXT_TARGET_NAMES = tuple([target[0] for target in TEXT_TARGETS])
@@ -78,7 +100,7 @@ PARSETREE_ACCEPT_TARGETS = (
         INTERNAL_PAGELIST_TARGET, PAGELIST_TARGET,
 ) + IMAGE_TARGETS + URI_TARGETS + TEXT_TARGETS
 PARSETREE_ACCEPT_TARGET_NAMES = tuple([target[0] for target in PARSETREE_ACCEPT_TARGETS])
-#~ print 'ACCEPT', PARSETREE_ACCEPT_TARGET_NAMES
+#~ print('ACCEPT', PARSETREE_ACCEPT_TARGET_NAMES)
 
 
 
@@ -94,19 +116,17 @@ PARSETREE_ACCEPT_TARGET_NAMES = tuple([target[0] for target in PARSETREE_ACCEPT_
 def pack_urilist(links):
 	text = ''
 	for link in links:
-		link = link.encode('utf-8')
 		if is_url_re.match(link):
 			link = url_encode(link, mode=URL_ENCODE_READABLE) # just to be sure
 		text += '%s\r\n' % link
-	return text
+	return text.encode()
 
 
 def unpack_urilist(text):
 	# FIXME be tolerant here for file://path/to/file uris here
-	text = text.strip('\x00') # Found trailing NULL character on windows
+	text = text.strip(b'\x00').decode() # Found trailing NULL character on windows
 	lines = text.splitlines() # takes care of \r\n
-	return [ line.decode('utf-8')
-		for line in lines if line and not line.isspace() ]
+	return [line for line in lines if line and not line.isspace()]
 		# Just to be sure we also skip empty or whitespace lines
 
 # TODO: Probably the serialize formats can replace custom copy/paste
@@ -114,42 +134,41 @@ def unpack_urilist(text):
 
 
 def textbuffer_register_serialize_formats(buffer, notebook, page):
-	if gtk.pygtk_version >= (2, 10):
-		buffer.register_serialize_format('text/x-zim-parsetree', serialize_parse_tree)
-		buffer.register_deserialize_format('text/x-zim-parsetree', deserialize_parse_tree, (notebook, page))
-		for name in (INTERNAL_PAGELIST_TARGET_NAME, PAGELIST_TARGET_NAME) + URI_TARGET_NAMES:
-			buffer.register_deserialize_format(name, deserialize_urilist, (notebook, page))
-		for name in IMAGE_TARGET_NAMES: # FIXME, should we limit the list ?
-			buffer.register_deserialize_format(name, deserialize_image, (name, notebook, page))
+	buffer.register_serialize_format('text/x-zim-parsetree', serialize_parse_tree)
+	buffer.register_deserialize_format('text/x-zim-parsetree', deserialize_parse_tree, (notebook, page))
+	for name in (INTERNAL_PAGELIST_TARGET_NAME, PAGELIST_TARGET_NAME) + URI_TARGET_NAMES:
+		buffer.register_deserialize_format(name, deserialize_urilist, (notebook, page))
+	for name in IMAGE_TARGET_NAMES: # FIXME, should we limit the list ?
+		buffer.register_deserialize_format(name, deserialize_image, (name, notebook, page))
 
-def serialize_parse_tree(register_buf, content_buf, start, end):
+def serialize_parse_tree(register_buf, content_buf, start, end, user_data):
 	tree = content_buf.get_parsetree((start, end))
-	xml = tree.tostring().encode('utf-8')
+	xml = tree.tostring()
 	return xml
 
-def deserialize_parse_tree(register_buf, content_buf, iter, data, create_tags, user_data):
+def deserialize_parse_tree(register_buf, content_buf, iter, data, length, create_tags, user_data):
 	notebook, path = user_data
 	tree = ParseTree().fromstring(data)
 	tree.resolve_images(notebook, path)
 	content_buf.insert_parsetree(iter, tree, interactive=True)
 	return True
 
-def deserialize_urilist(register_buf, content_buf, iter, data, create_tags, user_data):
+def deserialize_urilist(register_buf, content_buf, iter, data, length, create_tags, user_data):
 	notebook, path = user_data
 	links = unpack_urilist(data)
 	tree = _link_tree(links, notebook, path)
 	content_buf.insert_parsetree(iter, tree, interactive=True)
 	return True
 
-def deserialize_image(register_buf, content_buf, iter, data, create_tags, user_data):
+def deserialize_image(register_buf, content_buf, iter, data, length, create_tags, user_data):
 	# Implementation note: we follow gtk_selection_get_pixbuf() in usage of
-	# gtk.PixbufLoader to capture clipboard data in a pixbuf object.
+	# Gtk.PixbufLoader to capture clipboard data in a pixbuf object.
 	# We could skip this, but it allows for on-the-fly conversion of the data
 	# type.
 	mimetype, notebook, path = user_data
 
 	# capture image
-	loader = gtk.gdk.PixbufLoader()
+	loader = GdkPixbuf.PixbufLoader()
 	loader.write(data)
 	loader.close()
 	pixbuf = loader.get_pixbuf()
@@ -169,7 +188,7 @@ def deserialize_image(register_buf, content_buf, iter, data, create_tags, user_d
 
 	file = dir.new_file('pasted_image.%s' % extension)
 	logger.debug("Saving image from clipboard to %s", file)
-	pixbuf.save(file.path, format)
+	pixbuf.savev(file.path, format, [], [])
 	FS.emit('path-created', file) # notify version control
 
 	# and insert it in the page
@@ -192,7 +211,7 @@ def parsetree_from_selectiondata(selectiondata, notebook=None, path=None):
 	to save the image to the correct attachment folder and return a
 	parsetree with the correct image link.
 
-	@param selectiondata: a C{gtk.SelectionData} object
+	@param selectiondata: a C{Gtk.SelectionData} object
 	@param notebook: a L{Notebook} object
 	@param path: a L{Path} object
 
@@ -200,12 +219,12 @@ def parsetree_from_selectiondata(selectiondata, notebook=None, path=None):
 	'''
 	# TODO: check relative linking for all parsetrees !!!
 
-	targetname = str(selectiondata.target)
+	targetname = selectiondata.get_target().name()
 	if targetname == PARSETREE_TARGET_NAME:
-		return ParseTree().fromstring(selectiondata.data)
+		return ParseTree().fromstring(selectiondata.get_data())
 	elif targetname in (INTERNAL_PAGELIST_TARGET_NAME, PAGELIST_TARGET_NAME) \
 	or targetname in URI_TARGET_NAMES:
-		links = unpack_urilist(selectiondata.data)
+		links = selectiondata.get_uris()
 		return _link_tree(links, notebook, path)
 	elif targetname in TEXT_TARGET_NAMES:
 		# plain text parser should highlight urls etc.
@@ -213,7 +232,7 @@ def parsetree_from_selectiondata(selectiondata, notebook=None, path=None):
 		# try to catch this situation by a check here
 		text = selectiondata.get_text()
 		if text:
-			return get_format('plain').Parser().parse(text.decode('utf-8'), partial=True)
+			return get_format('plain').Parser().parse(text, partial=True)
 		else:
 			return None
 	elif targetname in IMAGE_TARGET_NAMES:
@@ -238,7 +257,7 @@ def parsetree_from_selectiondata(selectiondata, notebook=None, path=None):
 
 		file = dir.new_file('pasted_image.%s' % extension)
 		logger.debug("Saving image from clipboard to %s", file)
-		pixbuf.save(file.path, format)
+		pixbuf.savev(file.path, format, [], [])
 		FS.emit('path-created', file) # notify version control
 
 		links = [file.uri]
@@ -249,8 +268,8 @@ def parsetree_from_selectiondata(selectiondata, notebook=None, path=None):
 
 def _link_tree(links, notebook, path):
 	# Convert a list of links (of any type) into a parsetree
-	#~ print 'LINKS: ', links
-	#~ print 'NOTEBOOK and PATH:', notebook, path
+	#~ print('LINKS: ', links)
+	#~ print('NOTEBOOK and PATH:', notebook, path)
 	builder = ParseTreeBuilder()
 	builder.start(FORMATTEDTEXT)
 	for i in range(len(links)):
@@ -297,89 +316,105 @@ def _get_image_info(targetname):
 	# Target name for images is supposed to be mimetype, we check
 	# in available pixbuf writers for this type and return the
 	# format name and file extension
-	for format in gtk.gdk.pixbuf_get_formats():
-		if targetname == format['name'] \
-		or targetname == format['name'].upper() \
-		or targetname in format['mime_types']:
-			if format['is_writable']:
-				return format['name'], format['extensions'][0]
+	for format in GdkPixbuf.Pixbuf.get_formats():
+		name = format.get_name()
+		mimetypes = format.get_mime_types()
+		if targetname in (name, name.upper()) or targetname in mimetypes:
+			if format.is_writable():
+				return name, format.get_extensions()[0]
 			else:
 				return None, None
 	else:
 		return None, None
 
 
-class ClipboardItem(object):
-	'''Item that can be set on the clipboard'''
+class MockSelectionData(object):
+	'''Adapter to allow usage of C{ClipboardData} as input for
+	C{parsetree_from_selectiondata()}
+	'''
 
-	def set(self, clipboard, clear_func):
-		'''Put this item on the clipboard
-		@param clipboard: a C{gtk.Clipboard} objet
-		@param clear_func: function to be passed to the clipboard as
-		callback on clearing the data
+	def __init__(self, target, clipboard_data):
+		self.target = target
+		self.data = clipboard_data
+
+	def get_target(self):
+		target_name = self.target[0]
+		return Gdk.Atom.intern(target_name, False)
+
+	def get_data(self):
+		target_id = self.target[-1]
+		return self.data.get_data_as(target_id)
+
+	def get_text(self):
+		return self.data.get_data_as(TEXT_TARGET_ID)
+
+	def get_uris(self):
+		if self.target in URI_TARGETS:
+			return self.data.get_data_as(URI_TARGET_ID)
+		else:
+			return unpack_urilist(self.get_data())
+
+	def get_pixbuf(self):
+		raise NotImplementedError
+
+
+class ClipboardData(object):
+	'''Wrapper for data that can be set on the clipboard and pasted
+	multiple formats
+	'''
+
+	targets = ()
+
+	def get_data_as(self, targetid):
+		'''Return data in the requested target format
+		@param targetid: the target id
 		@implementation: must be implemented by sub-classes
 		'''
 		raise NotImplementedError
 
 
-class TextItem(ClipboardItem):
-	'''Text item for the clipboard'''
-
-	def __init__(self, text):
-		self.text = text
-
-	def set(self, clipboard, clear_func):
-		clipboard.set_text(self.text)
-
-
-class UriItem(ClipboardItem):
-	'''Uri item for the clipboard'''
+class UriData(ClipboardData):
 
 	targets = URI_TARGETS + TEXT_TARGETS
 
-	def __init__(self, obj):
-		if isinstance(obj, basestring):
-			self.uri = obj
-			self.obj = None
-		else:
-			assert hasattr(obj, 'uri')
-			self.obj = obj
-			self.uri = obj.uri
-
-	def set(self, clipboard, clear_func):
-		clipboard.set_with_data(self.targets, self._get, clear_func)
-
-	def _get(self, clipboard, selectiondata, id, *a):
-		if id == URI_TARGET_ID:
-			selectiondata.set_uris((self.uri,))
-		else:
-			if isinstance(self.obj, (File, Dir)):
-				selectiondata.set_text(self.obj.user_path)
+	def __init__(self, *obj):
+		uris = []
+		text = []
+		for o in obj:
+			if isinstance(o, (File, Dir)):
+				uris.append(o.uri)
+				text.append(o.user_path)
 			else:
-				selectiondata.set_text(self.uri)
+				uri = o if isinstance(o, str) else o.uri
+				uris.append(uri)
+				text.append(uri)
+		self.uris = tuple(uris)
+		self.text = ' '.join(text)
+
+	def get_data_as(self, targetid):
+		if targetid == URI_TARGET_ID:
+			return self.uris
+		else:
+			return self.text
 
 
-class InterWikiLinkItem(UriItem):
-	'''Like L{UriItem} but with special case for zim parsetree'''
+class InterWikiLinkData(UriData):
 
-	targets = (PARSETREE_TARGET,) + UriItem.targets
+	targets = (PARSETREE_TARGET,) + UriData.targets
 
 	def __init__(self, href, url):
-		UriItem.__init__(self, url)
+		UriData.__init__(self, url)
 		self.interwiki_href = href
 
-	def _get(self, clipboard, selectiondata, id, *a):
-		logger.debug("Clipboard requests data as '%s', we have an interwiki link", selectiondata.target)
-		if id == PARSETREE_TARGET_ID:
-			tree = _link_tree((self.interwiki_href,), None, None)
-			xml = tree.tostring().encode('utf-8')
-			selectiondata.set(PARSETREE_TARGET_NAME, 8, xml)
+	def get_data_as(self, targetid):
+		if targetid == PARSETREE_TARGET_ID:
+			parsetree = _link_tree((self.interwiki_href,), None, None)
+			return parsetree.tostring()
 		else:
-			UriItem._get(self, clipboard, selectiondata, id, *a)
+			return UriData.get_data_as(self, targetid)
 
 
-class ParseTreeItem(ClipboardItem):
-	'''Clipboard wrapper for a L{ParseTree}.'''
+class ParseTreeData(ClipboardData):
 
 	targets = (PARSETREE_TARGET,) + HTML_TARGETS + TEXT_TARGETS
 
@@ -389,47 +424,28 @@ class ParseTreeItem(ClipboardItem):
 		self.parsetree = parsetree
 		self.format = format
 
-	def set(self, clipboard, clear_func):
-		clipboard.set_with_data(self.targets, self._get, clear_func, self) \
-			or logger.warn('Failed to set data on clipboard')
-
-	def _get(self, clipboard, selectiondata, id, *a):
-		'''Callback to get the data in a specific format
-		@param clipboard: a C{gtk.Clipboard} objet
-		@param selectiondata: a C{gtk.SelectionData} object to set the data on
-		@param id: target id for the requested data format
-		@param a: any additional arguments are discarded
-		'''
-		logger.debug("Clipboard requests data as '%s', we have a parsetree", selectiondata.target)
-		if id == PARSETREE_TARGET_ID:
+	def get_data_as(self, targetid):
+		if targetid == PARSETREE_TARGET_ID:
 			# TODO make links absolute (?)
-			xml = self.parsetree.tostring().encode('utf-8')
-			selectiondata.set(PARSETREE_TARGET_NAME, 8, xml)
-		elif id == HTML_TARGET_ID:
+			return self.parsetree.tostring()
+		elif targetid == HTML_TARGET_ID:
 			dumper = get_format('html').Dumper(
-				linker=StaticExportLinker(self.notebook, source=self.path) )
-			html = ''.join( dumper.dump(self.parsetree) )
-			html = wrap_html(html, target=selectiondata.target)
-			#~ print 'PASTING: >>>%s<<<' % html
-			selectiondata.set(selectiondata.target, 8, html)
-		elif id == TEXT_TARGET_ID:
-			logger.debug("Clipboard requested text, we provide '%s'" % self.format)
-			#~ print ">>>>", self.format, parsetree.tostring()
-
+				linker=StaticExportLinker(self.notebook, source=self.path))
+			html = ''.join(dumper.dump(self.parsetree))
+			return wrap_html(html, target=selectiondata.get_target().name())
+		elif targetid == TEXT_TARGET_ID:
 			if self.format in ('wiki', 'plain'):
 				dumper = get_format(self.format).Dumper()
 			else:
 				dumper = get_format(self.format).Dumper(
-					linker=StaticExportLinker(self.notebook, source=self.path) )
+					linker=StaticExportLinker(self.notebook, source=self.path))
 
-			text = ''.join( dumper.dump(self.parsetree) ).encode('utf-8')
-			selectiondata.set_text(text)
+			return ''.join(dumper.dump(self.parsetree))
 		else:
-			assert False, 'Unknown target id %i' % id
+			raise ValueError('Unknown target id %i' % targetid)
 
 
-class PageLinkItem(ClipboardItem):
-	'''Clipboard wrapper for a L{ParseTree}.'''
+class PageLinkData(ClipboardData):
 
 	targets = (INTERNAL_PAGELIST_TARGET, PAGELIST_TARGET) + TEXT_TARGETS
 
@@ -437,65 +453,67 @@ class PageLinkItem(ClipboardItem):
 		self.notebookname = notebook.name
 		self.path = path
 
-	def set(self, clipboard, clear_func):
-		clipboard.set_with_data(self.targets, self._get, clear_func, self) \
-			or logger.warn('Failed to set data on clipboard')
-
-	def _get(self, clipboard, selectiondata, id, *a):
-		'''Callback to get the data in a specific format
-		@param clipboard: a C{gtk.Clipboard} objet
-		@param selectiondata: a C{gtk.SelectionData} object to set the data on
-		@param id: target id for the requested data format
-		@param a: any additional arguments are discarded
-		'''
-		logger.debug("Clipboard requests data as '%s', we have a pagelink", selectiondata.target)
-		if id == INTERNAL_PAGELIST_TARGET_ID:
-			text = pack_urilist((self.path.name,))
-			selectiondata.set(INTERNAL_PAGELIST_TARGET_NAME, 8, text)
-		elif id == PAGELIST_TARGET_ID:
+	def get_data_as(self, targetid):
+		if targetid == INTERNAL_PAGELIST_TARGET_ID:
+			return pack_urilist((self.path.name,))
+		elif targetid == PAGELIST_TARGET_ID:
 			link = "%s?%s" % (self.notebookname, self.path.name)
-			text = pack_urilist((link,))
-			selectiondata.set(PAGELIST_TARGET_NAME, 8, text)
-		elif id == TEXT_TARGET_ID:
-			selectiondata.set_text(self.path.name)
+			return pack_urilist((link,))
+		elif targetid == TEXT_TARGET_ID:
+			return self.path.name
 		else:
-			assert False, 'Unknown target id %i' % id
+			raise ValueError('Unknown target id %i' % targetid)
 
 
 class ClipboardManager(object):
-	'''Interface to the clipboard for copy and paste.
-	Replacement for gtk.Clipboard(), to be used everywhere in the
-	zim gui modules.
-	'''
+	'''Wrapper for C{Gtk.Clipboard}, supporting specific data formats'''
 
-	def __init__(self, atom):
+	def __init__(self, name):
 		'''Constructor
-		@param atom: clipboard name, can be either "CLIPBOARD" or "PRIMARY",
-		see C{gtk.Clipboard} for details.
+		@param name: clipboard name, can be either "CLIPBOARD" or "PRIMARY",
+		see C{Gtk.Clipboard} for details.
 		'''
-		assert atom in ('CLIPBOARD', 'PRIMARY')
-		self.clipboard = gtk.Clipboard(selection=atom)
-		self.store = None
-		self._i_am_owner = False
+		assert name in ('CLIPBOARD', 'PRIMARY')
+		atom = Gdk.SELECTION_CLIPBOARD if name == 'CLIPBOARD' else Gdk.SELECTION_PRIMARY
+		self.clipboard = Gtk.Clipboard.get(atom)
+		self.data = None
 
 	def clear(self):
-		'''Clear clipboard if we set it'''
-		if self._i_am_owner:
+		if self.data is not None:
 			self.clipboard.clear()
+		self.data = None
 
-	def set(self, item):
+	def set_clipboard_data(self, clipboard_data):
 		'''Set an item on the clipboard
-		@param item: a L{ClipboardItem}
+		@param clipboard_data: a L{ClipboardData} object
 		'''
-		item.set(self.clipboard, self._clear)
-		self._i_am_owner = True
-		if self.store:
-			pass
+		self.data = clipboard_data
+		#self.clipboard.set_with_data(self.data.targets, self._get, self._clear) \
+		#	or logger.warn('Failed to set data on clipboard')
 
-	def _clear(self, o, item):
-		self._i_am_owner = False
-		if self.store:
-			pass
+		### set_with_data() workaround
+		assert TEXT_TARGETS[0] in self.data.targets
+		text = self.data.get_data_as(TEXT_TARGET_ID)
+		self.clipboard.set_text(text, -1)
+		self.data._workaround_text = text
+		###
+
+	def _get(self, clipboard, selectiondata, targetid):
+		logger.debug(
+			"Clipboard requests data as '%s', we have %r",
+			selectiondata.get_target().name(),
+			self.data
+		)
+		data = self.data.get_data_as(targetid)
+		if targetid == TEXT_TARGET_ID:
+			selectiondata.set_text(data, -1)
+		elif targetid == URI_TARGET_ID:
+			selectiondata.set_uris(data)
+		else:
+			selectiondata.set(PARSETREE_TARGET_NAME, 8, data)
+
+	def _clear(self):
+		self.data = None
 
 	def set_text(self, text):
 		'''Set text to the clipboard
@@ -503,8 +521,7 @@ class ClipboardManager(object):
 		@note: DO NOT USE THIS METHOD if you can use L{set_parsetree()}
 		instead
 		'''
-		item = TextItem(text)
-		self.set(item)
+		self.clipboard.set_text(text, -1)
 
 	def get_text(self):
 		'''Get text from the clipboard.
@@ -512,10 +529,7 @@ class ClipboardManager(object):
 		@note: DO NOT USE THIS METHOD if you can use L{get_parsetree()}
 		instead
 		'''
-		text = self.clipboard.wait_for_text()
-		if isinstance(text, basestring):
-			text = text.decode('utf-8')
-		return text
+		return self.clipboard.wait_for_text()
 
 	def set_parsetree(self, notebook, path, parsetree, format='plain'):
 		'''Copy a parsetree to the clipboard. The parsetree can be pasted by
@@ -527,8 +541,8 @@ class ClipboardManager(object):
 		@param parsetree: the actual L{ParseTree} to be set on the clipboard
 		@keyword format: the format to use for pasting text, e.g. 'wiki' or 'plain'
 		'''
-		item = ParseTreeItem(notebook, path, parsetree, format)
-		self.set(item)
+		self.set_clipboard_data(
+			ParseTreeData(notebook, path, parsetree, format) )
 
 	def get_parsetree(self, notebook=None, path=None):
 		'''Get a parsetree from the clipboard.
@@ -545,19 +559,35 @@ class ClipboardManager(object):
 
 		@returns: a L{ParseTree} or C{None}
 		'''
-		targets = self.clipboard.wait_for_targets()
-		logger.debug('Targets available for paste: %s, we want parsetree', targets)
+		(has_data, atoms) = self.clipboard.wait_for_targets()
+		logger.debug('Targets available for paste: %s, we want parsetree', atoms)
 
-		if targets is None:
+		if not has_data:
 			return None
 
-		targets = [n for n in PARSETREE_ACCEPT_TARGET_NAMES if n in targets]
-			# Filter and sort by PARSETREE_ACCEPT_TARGET_NAMES
+		### set_with_data() workaround
+		if atoms and self.data and hasattr(self.data, '_workaround_text') \
+			and any(a.name() in TEXT_TARGET_NAMES for a in atoms):
+				text = self.clipboard.wait_for_text()
+				if text == self.data._workaround_text:
+					targets = sorted(
+						filter(lambda t: t in PARSETREE_ACCEPT_TARGETS, self.data.targets),
+						key=lambda t: PARSETREE_ACCEPT_TARGETS.index(t)
+					)
+					assert len(targets) > 0
+					logger.debug('Requesting data for %s -- using set_with_data() workaround', targets[0])
+					selectiondata = MockSelectionData(targets[0], self.data)
+					return parsetree_from_selectiondata(selectiondata, notebook, path)
+		###
 
-		if targets:
-			name = targets[0]
-			logger.debug('Requesting data for %s', name)
-			selectiondata = self.clipboard.wait_for_contents(name)
+		atoms = sorted(
+					filter(lambda a: a.name() in PARSETREE_ACCEPT_TARGET_NAMES, atoms),
+					key=lambda a: PARSETREE_ACCEPT_TARGET_NAMES.index(a.name())
+				)
+		if atoms:
+			atom = atoms[0]  # TODO why choose 1st index?
+			logger.debug('Requesting data for %s', atom)
+			selectiondata = self.clipboard.wait_for_contents(atom)
 			if selectiondata:
 				return parsetree_from_selectiondata(selectiondata, notebook, path)
 			else:
@@ -573,8 +603,7 @@ class ClipboardManager(object):
 		@param notebook: a L{Notebook} object
 		@param path: a L{Path} object
 		'''
-		item = PageLinkItem(notebook, path)
-		self.set(item)
+		self.set_clipboard_data(PageLinkData(notebook, path))
 
 	def set_interwikilink(self, href, url):
 		'''Copy an interwiki link to the clipboard
@@ -582,17 +611,18 @@ class ClipboardManager(object):
 		@param url: the expanded url for this interwiki link, e.g.
 		"http://en.wikipedia.org/wiki/foobar"
 		'''
-		item = InterWikiLinkItem(href, url)
-		self.set(item)
+		self.set_clipboard_data(InterWikiLinkData(href, url))
 
-	def set_uri(self, uri):
+	def set_uri(self, *uris):
 		'''Copy an uri to the clipboard
 		@param uri: an uri as string, or an object with an attribute C{uri}
 		'''
-		item = UriItem(uri)
-		self.set(item)
+		self.set_clipboard_data(UriData(*uris))
+
 
 Clipboard = ClipboardManager("CLIPBOARD") #: Singleton object for the default clipboard
+
+
 SelectionClipboard = ClipboardManager("PRIMARY") #: Singleton object for the selection clipboard (unix)
 
 
@@ -607,7 +637,6 @@ HTML_HEAD = '''\
 
 def wrap_html(html, target):
 	'''Function to wrap html with appropriate headers based on target type'''
-	html = html.encode('utf-8')
 	if target == 'HTML Format':
 		return Win32HtmlFormat.encode(html, head=HTML_HEAD)
 	else:
@@ -689,9 +718,9 @@ class Win32HtmlFormat:
 		lenPrefix = len(dummyPrefix)
 
 		prefix = klass.MARKER_BLOCK_OUTPUT % (
-			lenPrefix, len(html)+lenPrefix,
-			fragmentStart+lenPrefix, fragmentEnd+lenPrefix,
-			selectionStart+lenPrefix, selectionEnd+lenPrefix,
+			lenPrefix, len(html) + lenPrefix,
+			fragmentStart + lenPrefix, fragmentEnd + lenPrefix,
+			selectionStart + lenPrefix, selectionEnd + lenPrefix,
 			source
 		)
 		return prefix + html
