@@ -1,18 +1,25 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2009-2013 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
-from __future__ import with_statement
 
-import pango
-import gtk
+
+from gi.repository import Pango
+from gi.repository import Gtk
+from gi.repository import GObject
+
 import logging
 
-from zim.gui.widgets import Dialog, Button, BrowserTreeView, \
-	ScrolledWindow, ScrolledTextView, InputForm, input_table_factory
-from zim.gui.applications import CustomizeOpenWithDialog
+from zim.gui.widgets import Dialog, BrowserTreeView, \
+	ScrolledWindow, ScrolledTextView, InputForm, input_table_factory, get_window
+from zim.gui.applications import CustomizeOpenWithDialog, open_folder_prompt_create
 
 from zim.plugins import PLUGIN_FOLDER
+from zim.config import String, ConfigManager
+from zim.plugins import PluginManager
+from zim.main import ZIM_APPLICATION
+
+from zim.gui.mainwindow import ui_preferences as interface_preferences
+from zim.gui.pageview import ui_preferences as pageview_preferences
 
 
 logger = logging.getLogger('zim.gui.preferencesdialog')
@@ -24,84 +31,94 @@ _label = _('Editing') # T: Tab in preferences dialog
 
 
 class PreferencesDialog(Dialog):
-	'''Preferences dialog consisting of tabs with various options and
-	a tab with plugins. Options are not defined here, but need to be
-	registered using GtkInterface.register_preferences().
-	'''
 
-	def __init__(self, ui, default_tab=None, select_plugin=None):
-		Dialog.__init__(self, ui, _('Preferences')) # T: Dialog title
-		gtknotebook = gtk.Notebook()
-		self.vbox.add(gtknotebook)
+	def __init__(self, widget, default_tab=None, select_plugin=None):
+		Dialog.__init__(self, widget, _('Preferences')) # T: Dialog title
+		self.preferences = ConfigManager.preferences
+
 		# saves a list of loaded plugins to be used later
-		self.p_save_loaded = [p.__class__ for p in self.ui.plugins]
+		self.plugins = PluginManager()
+		self.p_save_loaded = list(self.plugins)
 
 		# Dynamic tabs
+		gtknotebook = Gtk.Notebook()
+		self.vbox.pack_start(gtknotebook, True, True, 0)
 		self.forms = {}
-		for category, preferences in ui.preferences_register.items():
-			vbox = gtk.VBox()
-			index = gtknotebook.append_page(vbox, gtk.Label(_(category)))
+
+		############################### needs rewrite to make defintion more robust
+		for category in ('Interface', 'Editing'):
+			vbox = Gtk.VBox()
+			index = gtknotebook.append_page(vbox, Gtk.Label(label=_(category)))
 			# From GTK Doc: Note that due to historical reasons, GtkNotebook refuses
 			# to switch to a page unless the child widget is visible.
 			vbox.show()
-			if category == default_tab: gtknotebook.set_current_page(index)
+			if category == default_tab:
+				gtknotebook.set_current_page(index)
 
 			fields = []
 			values = {}
 			sections = {}
-			for p in preferences:
-				if len(p) == 4:
-					section, key, type, label = p
-					fields.append((key, type, label))
-				else:
-					section, key, type, label, check = p
-					fields.append((key, type, label, check))
 
-				values[key] = ui.preferences[section][key]
-				sections[key] = section
+			for section, preferences in (
+				('GtkInterface', interface_preferences),
+				('PageView', pageview_preferences)
+			):
+				for p in [p for p in preferences if p[2] == category]:
+					# key, type, category, label, default, (check)
+					if len(p) == 5:
+						key, type, cat, label, default = p
+						self.preferences[section].setdefault(key, default)
+						fields.append((key, type, label))
+					else:
+						key, type, cat, label, default, check = p
+						self.preferences[section].setdefault(key, default, check)
+						fields.append((key, type, label, check))
+
+					values[key] = self.preferences[section][key]
+					sections[key] = section
 
 			form = InputForm(fields, values)
 			form.preferences_sections = sections
-			vbox.pack_start(form, False)
+			vbox.pack_start(form, False, True, 0)
 			self.forms[category] = form
 
 			if category == 'Interface':
 				self._add_font_selection(form)
 
 		# Styles tab
-		#~ gtknotebook.append_page(StylesTab(self), gtk.Label(_('Styles')))
+		#~ gtknotebook.append_page(StylesTab(self), Gtk.Label(label=_('Styles')))
 
 		# Keybindings tab
-		#~ gtknotebook.append_page(KeyBindingsTab(self), gtk.Label(_('Key bindings')))
+		#~ gtknotebook.append_page(KeyBindingsTab(self), Gtk.Label(label=_('Key bindings')))
 
 		# Plugins tab
-		plugins_tab = PluginsTab(self, self.ui.plugins)
-		plugins_tab_index = gtknotebook.append_page(plugins_tab, gtk.Label(_('Plugins')))
+		self.plugins_tab = PluginsTab(self, self.plugins)
+		plugins_tab_index = gtknotebook.append_page(self.plugins_tab, Gtk.Label(label=_('Plugins')))
 				# T: Heading in preferences dialog
-		plugins_tab.show()
+		self.plugins_tab.show()
 		#~ print default_tab, index
 		if default_tab == "Plugins":
 			gtknotebook.set_current_page(plugins_tab_index)
 			if not select_plugin is None:
-					plugins_tab.select_plugin(select_plugin)
+					self.plugins_tab.select_plugin(select_plugin)
 
 		# Applications tab
-		gtknotebook.append_page(ApplicationsTab(self), gtk.Label(_('Applications')))
+		gtknotebook.append_page(ApplicationsTab(self), Gtk.Label(label=_('Applications')))
 			# T: Heading in preferences dialog
 
 
 	def _add_font_selection(self, table):
 		# need to hardcode this, cannot register it as a preference
-		table.add_inputs( (
+		table.add_inputs((
 			('use_custom_font', 'bool', _('Use a custom font')),
 			# T: option in preferences dialog
-		) )
+		))
 		table.preferences_sections['use_custom_font'] = 'GtkInterface'
 
-		self.fontbutton = gtk.FontButton()
+		self.fontbutton = Gtk.FontButton()
 		self.fontbutton.set_use_font(True) # preview in button
 		self.fontbutton.set_sensitive(False)
-		text_style = self.ui.config.get_config_dict('<profile>/style.conf')
+		text_style = ConfigManager.get_config_dict('style.conf')
 		try:
 			font = text_style['TextView']['font']
 			if font:
@@ -112,7 +129,7 @@ class PreferencesDialog(Dialog):
 			pass
 
 		table.widgets['use_custom_font'].connect('toggled',
-			lambda o: self.fontbutton.set_sensitive(o.get_active()) )
+			lambda o: self.fontbutton.set_sensitive(o.get_active()))
 
 		self.fontbutton.set_size_request(100, -1)
 		input_table_factory(((None, self.fontbutton),), table)
@@ -120,8 +137,8 @@ class PreferencesDialog(Dialog):
 	def do_response_ok(self):
 		# Get dynamic tabs
 		newpreferences = {}
-		for form in self.forms.values():
-			for key, value in form.items():
+		for form in list(self.forms.values()):
+			for key, value in list(form.items()):
 				section = form.preferences_sections[key]
 				if not section in newpreferences:
 					newpreferences[section] = {}
@@ -134,54 +151,50 @@ class PreferencesDialog(Dialog):
 		else:
 			font = None
 
-		text_style = self.ui.config.get_config_dict('<profile>/style.conf')
+		text_style = ConfigManager.get_config_dict('style.conf')
+		text_style['TextView'].define(font=String(None))
 		text_style['TextView']['font'] = font
 		#
 
-		with self.ui.preferences.block_signals('changed'):
+		with self.preferences.block_signals('changed'):
 			# note we do not block signal on section dicts
 			for section in newpreferences:
-				self.ui.preferences[section].update(newpreferences[section])
+				self.preferences[section].update(newpreferences[section])
 
-		self.ui.preferences.emit('changed') # delayed emission
+		self.preferences.emit('changed') # delayed emission
 
 		return True
 
 	def do_response_cancel(self):
-		# TODO FIXME
-
 		# Obtain an updated list of loaded plugins
-		now_loaded = [p.__class__ for p in self.ui.plugins]
+		now_loaded = list(self.plugins)
 
 		# Restore previous situation if the user changed something
 		# in this dialog session
-		with self.ui.preferences.block_signals('changed'):
-			for name in self.ui.plugins.list_installed_plugins():
-				try:
-					klass = self.ui.plugins.get_plugin_class(name)
-				except:
-					continue
+		with self.preferences.block_signals('changed'):
+			for name in self.plugins.list_installed_plugins():
+				if name in self.p_save_loaded and name not in now_loaded:
+					try:
+						self.plugins.load_plugin(name)
+					except:
+						logger.exception('Could not restore plugin: %s', name)
+				elif name not in self.p_save_loaded and name in now_loaded:
+					self.plugins.remove_plugin(name)
 
-				activatable = klass.check_dependencies_ok()
-
-				if klass in self.p_save_loaded and activatable and klass not in now_loaded:
-					self.ui.load_plugin(klass.plugin_key)
-				elif klass not in self.p_save_loaded and klass in now_loaded:
-					self.ui.unload_plugin(klass.plugin_key)
-
-		self.ui.preferences.emit('changed') # delayed emission
+		self.preferences.emit('changed') # delayed emission
 
 		return True
 
 
-class PluginsTab(gtk.VBox):
+class PluginsTab(Gtk.VBox):
 
 	def __init__(self, dialog, plugins):
-		gtk.VBox.__init__(self, spacing=5)
+		GObject.GObject.__init__(self)
+		self.set_spacing(5)
 		self.dialog = dialog
 		self.plugins = plugins
 
-		self.hbox = gtk.HBox(self, spacing=12)
+		self.hbox = Gtk.HBox(self, spacing=12)
 		self.hbox.set_border_width(5)
 		self.add(self.hbox)
 
@@ -190,32 +203,32 @@ class PluginsTab(gtk.VBox):
 		self.treeview = PluginsTreeView(self.plugins)
 		self.treeselection = self.treeview.get_selection()
 		self.treeselection.connect('changed', self.do_selection_changed)
-		swindow = ScrolledWindow(self.treeview, hpolicy=gtk.POLICY_NEVER)
-		self.hbox.pack_start(swindow, False)
+		swindow = ScrolledWindow(self.treeview, hpolicy=Gtk.PolicyType.NEVER)
+		self.hbox.pack_start(swindow, False, True, 0)
 
-		vbox = gtk.VBox()
+		vbox = Gtk.VBox()
 		self.hbox.add(vbox)
 
 		# Textview with scrollbars to show plugins info. Required by small screen devices
 		swindow, textview = ScrolledTextView()
 		textview.set_cursor_visible(False)
 		self.textbuffer = textview.get_buffer()
-		self.textbuffer.create_tag('bold', weight=pango.WEIGHT_BOLD)
+		self.textbuffer.create_tag('bold', weight=Pango.Weight.BOLD)
 		self.textbuffer.create_tag('red', foreground='#FF0000')
-		vbox.pack_start(swindow, True)
+		vbox.pack_start(swindow, True, True, 0)
 
-		hbox = gtk.HBox(spacing=5)
-		vbox.pack_end(hbox, False)
+		hbox = Gtk.HBox(spacing=5)
+		vbox.pack_end(hbox, False, True, 0)
 
 		self.plugin_help_button = \
-			Button(stock=gtk.STOCK_HELP, label=_('_More')) # T: Button in plugin tab
+			Gtk.Button.new_with_mnemonic(_('_More')) # T: Button in plugin tab
 		self.plugin_help_button.connect('clicked', self.on_help_button_clicked)
-		hbox.pack_start(self.plugin_help_button, False)
+		hbox.pack_start(self.plugin_help_button, False, True, 0)
 
 		self.configure_button = \
-			Button(stock=gtk.STOCK_PREFERENCES, label=_('C_onfigure')) # T: Button in plugin tab
+			Gtk.Button.new_with_mnemonic(_('C_onfigure')) # T: Button in plugin tab
 		self.configure_button.connect('clicked', self.on_configure_button_clicked)
-		hbox.pack_start(self.configure_button, False)
+		hbox.pack_start(self.configure_button, False, True, 0)
 
 		try:
 			self.treeselection.select_path(0)
@@ -223,25 +236,23 @@ class PluginsTab(gtk.VBox):
 			pass # maybe loading plugins failed
 
 		## Add buttons to get and install new plugins
-		hbox = gtk.HButtonBox()
+		hbox = Gtk.HButtonBox()
 		hbox.set_border_width(5)
-		hbox.set_layout(gtk.BUTTONBOX_START)
-		self.pack_start(hbox, False)
+		hbox.set_layout(Gtk.ButtonBoxStyle.START)
+		self.pack_start(hbox, False, True, 0)
 
-		assert hasattr(self.dialog, 'ui')
-		open_button = gtk.Button(label=_('Open plugins folder'))
+		open_button = Gtk.Button.new_with_mnemonic(_('Open plugins folder'))
+			# T: button label
 		open_button.connect('clicked',
-			lambda o: self.dialog.ui.open_dir(PLUGIN_FOLDER)
+			lambda o: open_folder_prompt_create(o, PLUGIN_FOLDER)
 		)
-		hbox.pack_start(open_button, False)
+		hbox.pack_start(open_button, False, True, 0)
 
-		if gtk.gtk_version >= (2, 10) \
-		and gtk.pygtk_version >= (2, 10):
-			url_button = gtk.LinkButton(
-				'https://github.com/jaap-karssenberg/zim-wiki/wiki/Plugins',
-				_('Get more plugins online') # T: label for button with URL
-			)
-			hbox.pack_start(url_button, False)
+		url_button = Gtk.LinkButton(
+			'http://zim-wiki.org/more_plugins.html',
+			_('Get more plugins online') # T: label for button with URL
+		)
+		hbox.pack_start(url_button, False, True, 0)
 
 
 	def do_selection_changed(self, selection):
@@ -279,14 +290,14 @@ class PluginsTab(gtk.VBox):
 			for dependency in dependencies:
 				text, ok, required = dependency
 				if ok:
-					insert(u'\u2022 %s - %s\n' % (text, _('OK'))) # T: dependency is OK
+					insert('\u2022 %s - %s\n' % (text, _('OK'))) # T: dependency is OK
 				elif required:
-					insert(u'\u2022 %s - %s\n' % (text, _('Failed')), 'red') # T: dependency failed
+					insert('\u2022 %s - %s\n' % (text, _('Failed')), 'red') # T: dependency failed
 				else:
-					insert(u'\u2022 %s - %s (%s)\n' % (text,
+					insert('\u2022 %s - %s (%s)\n' % (text,
 						_('Failed'), # T: dependency failed
 						_('Optional') # T: optional dependency
-					) )
+					))
 		insert('\n')
 
 		insert(_('Author') + '\n', 'bold') # T: Heading in plugins tab of preferences dialog
@@ -297,7 +308,9 @@ class PluginsTab(gtk.VBox):
 
 	def on_help_button_clicked(self, button):
 		klass = self.plugins.get_plugin_class(self._current_plugin)
-		self.dialog.ui.show_help(klass.plugin_info['help']) # XXX
+		page = klass.plugin_info['help']
+		if page:
+			ZIM_APPLICATION.run('--manual', page)
 
 	def on_configure_button_clicked(self, button):
 		plugin = self.plugins[self._current_plugin]
@@ -306,20 +319,20 @@ class PluginsTab(gtk.VBox):
 	def select_plugin(self, name):
 		model = self.treeview.get_model()
 		def find(model, path, iter):
-			if model[iter][2] == name:
+			if name in model[iter]: # either key or localized name
 				self.treeview.scroll_to_cell(path)
 				self.treeview.set_cursor(path)
-				self.do_selection_changed(self.treeselection, 0)
-				return True;
+				self.do_selection_changed(self.treeselection)
+				return True
 			return False # keep the foreach going
 		model.foreach(find)
 
 
-class PluginsTreeModel(gtk.ListStore):
+class PluginsTreeModel(Gtk.ListStore):
 
 	def __init__(self, plugins):
 		#columns are: key, active, activatable, name, klass
-		gtk.ListStore.__init__(self, str, bool, bool, str, object)
+		Gtk.ListStore.__init__(self, str, bool, bool, str, object)
 		self.plugins = plugins
 
 		allplugins = []
@@ -368,13 +381,13 @@ class PluginsTreeView(BrowserTreeView):
 		model = PluginsTreeModel(plugins)
 		self.set_model(model)
 
-		cellrenderer = gtk.CellRendererToggle()
+		cellrenderer = Gtk.CellRendererToggle()
 		cellrenderer.connect('toggled', lambda o, p: model.do_toggle_path(p))
 		self.append_column(
-			gtk.TreeViewColumn(_('Enabled'), cellrenderer, active=1, activatable=2))
+			Gtk.TreeViewColumn(_('Enabled'), cellrenderer, active=1, activatable=2))
 			# T: Column in plugin tab
 		self.append_column(
-			gtk.TreeViewColumn(_('Plugin'), gtk.CellRendererText(), text=3))
+			Gtk.TreeViewColumn(_('Plugin'), Gtk.CellRendererText(), text=3))
 			# T: Column in plugin tab
 
 
@@ -384,33 +397,32 @@ class PluginConfigureDialog(Dialog):
 		Dialog.__init__(self, dialog, _('Configure Plugin')) # T: Dialog title
 		self.plugin = plugin
 
-		label = gtk.Label()
+		label = Gtk.Label()
 		label.set_markup(
-			'<b>'+_('Options for plugin %s') % plugin.plugin_info['name']+'</b>')
+			'<b>' + _('Options for plugin %s') % plugin.plugin_info['name'] + '</b>')
 			# T: Heading for 'configure plugin' dialog - %s is the plugin name
 		self.vbox.add(label)
 
-		fields = []
 		ignore = getattr(self.plugin, 'hide_preferences', [])
-		for pref in self.plugin.plugin_preferences:
-			if pref[0] in ignore:
-				continue
-
-			if len(pref) == 4:
-				key, type, label, default = pref
-				check = None
-				self.plugin.preferences.setdefault(key, default) # just to be sure
-			else:
-				key, type, label, default, check = pref
-				self.plugin.preferences.setdefault(key, default, check=check) # just to be sure
-
-			if type in ('int', 'choice'):
-				fields.append((key, type, label, check))
-			else:
-				fields.append((key, type, label))
-
+		fields = [
+			field for field in
+				self.plugin.form_fields(self.plugin.plugin_preferences)
+					if field[0] not in ignore
+		]
 		self.add_form(fields, self.plugin.preferences)
 
+		if plugin.plugin_notebook_properties:
+			hbox = Gtk.Box(spacing=12)
+			hbox.add(Gtk.Image.new_from_icon_name('dialog-information', Gtk.IconSize.DIALOG))
+			label = Gtk.Label()
+			label.set_markup(
+				'<i>' +
+				_('This plugin also has properties,\nsee the notebook properties dialog') + # T: info text in the preferences dialog
+				'</i>'
+			)
+			hbox.add(label)
+			self.vbox.pack_start(hbox, False, False, 18)
+
 	def do_response_ok(self):
 		# First let the plugin receive the changes, then save them.
 		# The plugin could do some conversion on the fly (e.g. Path to string)
@@ -418,43 +430,43 @@ class PluginConfigureDialog(Dialog):
 		return True
 
 
-class ApplicationsTab(gtk.VBox):
+class ApplicationsTab(Gtk.VBox):
 
 	def __init__(self, dialog):
-		gtk.VBox.__init__(self)
+		GObject.GObject.__init__(self)
 		self.set_border_width(5)
 		self.dialog = dialog
 
-		button = gtk.Button(_('Set default text editor'))
+		button = Gtk.Button.new_with_mnemonic(_('Set default text editor'))
 			# T: button in preferences dialog to change default text editor
 		button.connect('clicked', self.on_set_texteditor)
 
-		self.pack_start(button, False)
+		self.pack_start(button, False, True, 0)
 
 	def on_set_texteditor(self, o):
 		CustomizeOpenWithDialog(self.dialog, 'text/plain').run()
 
 
-class StylesTab(gtk.VBox):
+class StylesTab(Gtk.VBox):
 
 	def __init__(self, dialog):
-		gtk.VBox.__init__(self)
-		self.add(gtk.Label('TODO add treeview with styles'))
+		GObject.GObject.__init__(self)
+		self.add(Gtk.Label(label='TODO add treeview with styles'))
 
 
-class StylesTreeModel(gtk.ListStore):
+class StylesTreeModel(Gtk.ListStore):
 
-	def __init__(self, ui):
+	def __init__(self):
 		#'weight', 'scale', 'style', 'background', 'foreground', 'strikethrough',
 		# 'family', 'wrap-mode', 'indent', 'underline'
-		gtk.ListStore.__init__(self, bool, str, object)
+		Gtk.ListStore.__init__(self, bool, str, object)
 
 
-class KeyBindingsTab(gtk.VBox):
+class KeyBindingsTab(Gtk.VBox):
 
 	def __init__(self, dialog):
-		gtk.VBox.__init__(self)
-		self.add(gtk.Label('TODO add treeview with accelerators'))
+		GObject.GObject.__init__(self)
+		self.add(Gtk.Label(label='TODO add treeview with accelerators'))
 
 #~ Build editable treeview of menu items + accelerators
 #~
@@ -473,21 +485,21 @@ class KeyBindingsTab(gtk.VBox):
 #~
 #~ To get the accelerator:
 #~ accel_path = menuitem.get_accel_path() (make sure this is not the mnemonic..)
-#~ key, mod = gtk.accel_map_lookup_entry(accel_path)
+#~ key, mod = Gtk.AccelMap.lookup_entry(accel_path)
 #~
 #~ To get / set accelerator labels in the UI use:
-#~ gtk.accelerator_name() to get a name to display
+#~ Gtk.accelerator_name() to get a name to display
 #~
 #~ To parse name set by user
-#~ gtk.accelerator_parse()
-#~ gtk.accelerator_valid()
+#~ Gtk.accelerator_parse()
+#~ Gtk.accelerator_valid()
 #~
 #~ To change the accelerator:
 #~ Maybe first unlock path in accel_map and unlock the actiongroup..
-#~ gtk.accel_map.change_entry(accel_path, key, mods, replace=True)
+#~ Gtk.accel_map.change_entry(accel_path, key, mods, replace=True)
 #~ check return value
 #~
 #~ To get updates for ui use:
-#~ gtk.accel_map_get().connect('changed', func(o, accel_path, key, mods))
+#~ Gtk.AccelMap.get().connect('changed', func(o, accel_path, key, mods))
 #~ This way we also get any accelerators that were deleted as result of
 #~ replace=True
