@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2013-2016 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
@@ -45,7 +44,7 @@ class Command(object):
 		#: For options that can appear multiple times,
 		#: assign a list "[]" in "self.opts" before parse_options is called
 
-	default_options	 = (
+	default_options = (
 		('verbose', 'V', 'Verbose output'),
 		('debug', 'D', 'Debug output'),
 	)
@@ -105,9 +104,9 @@ class Command(object):
 		'''
 		minimum = len([a for a in self.arguments if not a.startswith('[')])
 		if len(self.args) < minimum:
-			raise UsageError, 'Command %s takes %i arguments' % (self.command, minimum)
+			raise UsageError('Command %s takes %i arguments' % (self.command, minimum))
 		elif len(self.args) > len(self.arguments):
-			raise UsageError, 'Command %s takes only %i arguments' % (self.command, len(self.args))
+			raise UsageError('Command %s takes only %i arguments' % (self.command, len(self.args)))
 		else:
 			return tuple(self.args) \
 				+ (None,) * (len(self.arguments) - len(self.args))
@@ -131,17 +130,17 @@ class GtkCommand(Command):
 	Gtk user interface.
 
 	If the C{run()} method returns a window, it will be added to the
-	application top level windows. And a C{gtk.main} loop will run untill
+	application top level windows. And a C{Gtk.main} loop will run untill
 	all windows are destroyed.
 
 	Commands derived from this class can be dispatched to the main application
 	process. This is controlled by the C{standalone_process} property.
 
-	NOTE: Do _not_ call C{gtk.main} from the command, this will be
+	NOTE: Do _not_ call C{Gtk.main} from the command, this will be
 	done by the application object.
 	'''
 
-	default_options	 = Command.default_options + (
+	default_options = Command.default_options + (
 		('standalone', '', 'start a single instance, no background process'),
 	)
 
