@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2015-2016 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
@@ -14,10 +13,9 @@ import logging
 logger = logging.getLogger('zim.newfs')
 
 
-from . import FS_ENCODING, FS_SUPPORT_NON_LOCAL_FILE_SHARES
+from . import FS_SUPPORT_NON_LOCAL_FILE_SHARES
 
 from zim.errors import Error
-from zim.environ import environ
 from zim.parsing import url_encode
 
 
@@ -25,9 +23,13 @@ is_url_re = re.compile('^\w{2,}:/')
 is_share_re = re.compile(r'^\\\\\w')
 
 
-_SEP = os.path.sep
+if os.name == 'nt':
+	SEP = '\\' # os.path.sep can still be "/" under msys
+	_EOL = 'dos'
+else:
+	SEP = os.path.sep
+	_EOL = 'unix'
 
-_EOL = 'dos' if os.name == 'nt' else 'unix'
 
 
 
@@ -63,7 +65,7 @@ class FileUnicodeError(Error):
 			# T: message for FileUnicodeError (%s is the file name)
 		self.description = _('This usually means the file contains invalid characters')
 			# T: message for FileUnicodeError
-		self.description += '\n\n' + _('Details') + ':\n' + unicode(error)
+		self.description += '\n\n' + _('Details') + ':\n' + str(error)
 			# T: label for detailed error
 
 
@@ -100,7 +102,7 @@ class FolderNotEmptyError(Error):
 def _split_file_url(url):
 	scheme, path = url.replace('\\', '/').split(':/', 1)
 	if scheme not in ('file', 'smb'):
-		raise ValueError, 'Not a file URL: %s' % url
+		raise ValueError('Not a file URL: %s' % url)
 
 	if path.startswith('/localhost/'): # exact 2 '/' before 'localhost'
 		path = path[11:]
@@ -113,10 +115,12 @@ def _split_file_url(url):
 	return path.strip('/').split('/'), isshare
 
 
-def _splitnormpath(path):
+def _splitnormpath(path, force_rel=False):
 	# Takes either string or list of names and returns a normalized tuple
 	# Keeps leading "/" or "\\" to distinguish absolute paths
-	if isinstance(path, basestring):
+	# Split must be robust for both "/" and "\" pathseperators regardless of
+	# the os we are running on !
+	if isinstance(path, str) and not force_rel:
 		if is_url_re.match(path):
 			makeroot = True
 			path, makeshare = _split_file_url(path)
@@ -131,6 +135,8 @@ def _splitnormpath(path):
 	else:
 		makeshare = False
 		makeroot = False
+		if isinstance(path, str):
+			path = re.split(r'[/\\]+', path.strip('/\\'))
 
 	names = []
 	for name in path:
@@ -146,10 +152,10 @@ def _splitnormpath(path):
 			names.append(name)
 
 	if not names:
-		raise ValueError, 'path reduces to empty string'
+		raise ValueError('path reduces to empty string')
 	elif makeshare:
 		names[0] = '\\\\' + names[0] # UNC host needs leading "\\"
-	elif makeroot and os.name != 'nt' and names[0][0] != '/':
+	elif makeroot and os.name != 'nt' and not names[0].startswith('/'):
 		names[0] = '/' + names[0]
 
 	return tuple(names)
@@ -159,14 +165,14 @@ if os.name == 'nt':
 	def _joinabspath(names):
 		# first element must be either drive letter or UNC host
 		if not re.match(r'^(\w:|\\\\\w)', names[0]):
-			raise ValueError, 'Not an absolute path: %s' % '\\'.join(names)
+			raise ValueError('Not an absolute path: %s' % '\\'.join(names))
 		else:
-			return '\\'.join(names)
+			return '\\'.join(names) # Don't rely on SEP here, msys sets it to '/'
 
 	def _joinuri(names):
 		# first element must be either drive letter or UNC host
 		if not re.match(r'^(\w:|\\\\\w)', names[0]):
-			raise ValueError, 'Not an absolute path: %s' % '\\'.join(names)
+			raise ValueError('Not an absolute path: %s' % '\\'.join(names))
 		elif re.match(r'^\w:$', names[0]): # Drive letter - e.g. file:///C:/foo
 			return 'file:///' + names[0] + '/' + url_encode('/'.join(names[1:]))
 		elif re.match(r'^\\\\\w+$', names[0]): # UNC path - e.g. file://host/share
@@ -179,7 +185,7 @@ else:
 		elif names[0].startswith('/'):
 			return '/'.join(names)
 		else:
-			raise ValueError, 'Not an absolute path: %s' % '/'.join(names)
+			raise ValueError('Not an absolute path: %s' % '/'.join(names))
 
 	def _joinuri(names):
 		if names[0][0] == '/':
@@ -188,73 +194,19 @@ else:
 			return 'file:///' + url_encode('/'.join(names))
 
 
-
-if FS_ENCODING == 'mbcs':
-	# Encoding 'mbcs' means we run on windows and filesystem can handle utf-8 natively
-	# so here we just convert everything to unicode strings
-	def _encode_path(path):
-		return path if isinstance(path, unicode) else unicode(path)
-
-	_decode_path = _encode_path
-
-else:
-	# Here we encode files to filesystem encoding. Fails if encoding is not possible.
-	def _encode_path(path):
-		if isinstance(path, unicode):
-			try:
-				return path.encode(FS_ENCODING)
-			except UnicodeEncodeError:
-				raise ValueError, 'BUG: invalid filename %s' % path
-				#~ raise Error, 'BUG: invalid filename %s' % path
-		else:
-			return path # assume encoding is correct
-
-	def _decode_path(path):
-		if isinstance(path, unicode):
-			return path # assume encoding is correct
-		else:
-			try:
-				return path.decode(FS_ENCODING)
-			except UnicodeDecodeError:
-				raise ValueError, 'BUG: invalid filename %s' % path
-				#~ raise Error, 'BUG: invalid filename %s' % path
-
-
-
 def _os_expanduser(path):
-	'''Wrapper for C{os.path.expanduser()} to get encoding right'''
 	assert path.startswith('~')
-	if FS_ENCODING == 'mbcs':
-		# This method is an exception in that it does not handle unicode
-		# directly. This will cause and error when user name contains
-		# non-ascii characters. See bug report lp:988041.
-		# But also mbcs encoding does not handle all characters,
-		# so only encode home part
-		parts = path.replace('\\', '/').strip('/').split('/')
-			# parts[0] now is "~" or "~user"
-
-		if isinstance(path, unicode):
-			part = parts[0].encode('mbcs')
-			part = os.path.expanduser(part)
-			parts[0] = part.decode('mbcs')
-		else:
-			# assume it is compatible
-			parts[0] = os.path.expanduser(parts[0])
-
-		path = _SEP.join(parts)
-	else:
-		# Let encode() handle the unicode encoding
-		path = _decode_path(os.path.expanduser(_encode_path(path)))
+	path = os.path.expanduser(path)
 
 	if path.startswith('~'):
 		# expansion failed - do a simple fallback
-		home = environ['HOME']
+		home = os.environ['HOME']
 		parts = path.replace('\\', '/').strip('/').split('/')
 		if parts[0] == '~':
-			path = _SEP.join([home] + parts[1:])
+			path = SEP.join([home] + parts[1:])
 		else: # ~user
 			dir = os.path.dirname(home) # /home or similar ?
-			path = _SEP.join([dir, parts[0][1:]] + parts[1:])
+			path = SEP.join([dir, parts[0][1:]] + parts[1:])
 
 	return path
 
@@ -276,14 +228,14 @@ class FilePath(object):
 	__slots__ = ('path', 'pathnames', 'islocal')
 
 	def __init__(self, path):
-		if isinstance(path, (tuple, list, basestring)):
+		if isinstance(path, (tuple, list, str)):
 			self.pathnames = _splitnormpath(path)
 			self.path = _joinabspath(self.pathnames)
 		elif isinstance(path, FilePath):
 			self.pathnames = path.pathnames
 			self.path = path.path
 		else:
-			raise TypeError, 'Cannot convert %r to a FilePath' % path
+			raise TypeError('Cannot convert %r to a FilePath' % path)
 
 		self.islocal = not self.pathnames[0].startswith('\\\\')
 
@@ -325,15 +277,15 @@ class FilePath(object):
 	@property
 	def userpath(self):
 		if self.ischild(_HOME):
-			return '~' + _SEP + self.relpath(_HOME)
+			return '~' + SEP + self.relpath(_HOME)
 		else:
 			return self.path
 
 	def get_childpath(self, path):
 		assert path
-		names = _splitnormpath(path)
+		names = _splitnormpath(path, force_rel=True)
 		if not names or names[0] == '..':
-			raise ValueError, 'Relative path not below parent: %s' % path
+			raise ValueError('Relative path not below parent: %s' % path)
 		return FilePath(self.pathnames + names)
 
 	def get_abspath(self, path):
@@ -358,15 +310,15 @@ class FilePath(object):
 		if allowupward and not self.ischild(start):
 			parent = self.commonparent(start)
 			if parent is None:
-				raise ValueError, 'No common parent between %s and %s' % (self.path, start.path)
+				raise ValueError('No common parent between %s and %s' % (self.path, start.path))
 			relpath = self.relpath(parent)
 			level_up = len(start.pathnames) - len(parent.pathnames)
-			return (('..' + _SEP) * level_up) + relpath
+			return (('..' + SEP) * level_up) + relpath
 		else:
 			names = start.pathnames
 			if not self.pathnames[:len(names)] == names:
-				raise ValueError, 'Not a parent path: %s' % start.path
-			return _SEP.join(self.pathnames[len(names):])
+				raise ValueError('Not a parent path: %s' % start.path)
+			return SEP.join(self.pathnames[len(names):])
 
 	def commonparent(self, other):
 		if self.pathnames[0] != other.pathnames[0]:
@@ -377,21 +329,37 @@ class FilePath(object):
 			return self
 		else:
 			for i in range(1, len(self.pathnames)):
-				if self.pathnames[:i+1] != other.pathnames[:i+1]:
+				if self.pathnames[:i + 1] != other.pathnames[:i + 1]:
 					return FilePath(self.pathnames[:i])
 
 
 
 _HOME = FilePath('~')
 
+class FSObjectMeta(type):
+	'''This meta class allows implementing wrappers for file and folder objects
+	with C{isinstance()} checking the wrapped class as well as the wrapper.
+	Main use case is filtered version of folder object where e.g.
+	C{isinstance(folder, LocalFolder)} is used to check whether the underlying
+	resources exist external to the application.
+	'''
 
-class FSObjectBase(FilePath):
+	def __instancecheck__(cls, instance):
+		if instance.__class__ == cls or issubclass(instance.__class__, cls):
+			return True
+		elif hasattr(instance, '_inner_fs_object') and isinstance(instance._inner_fs_object, cls):
+			return True
+		else:
+			return False
+
+
+class FSObjectBase(FilePath, metaclass=FSObjectMeta):
 	'''Base class for L{File} and L{Folder}'''
 
 	def __init__(self, path, watcher=None):
 		FilePath.__init__(self, path)
 		if not FS_SUPPORT_NON_LOCAL_FILE_SHARES and not self.islocal:
-			raise ValueError, 'File system does not support non-local files'
+			raise ValueError('File system does not support non-local files')
 
 		self.watcher = watcher
 
@@ -437,7 +405,7 @@ class FSObjectBase(FilePath):
 		self._copyto(other)
 		self.remove()
 
-	def remove(self):
+	def remove(self, cleanup=True):
 		raise NotImplementedError
 
 	def _cleanup(self):
@@ -454,18 +422,24 @@ class Folder(FSObjectBase):
 	'''
 
 	def __init__(self, path):
-		raise NotImplementedError, 'This class is not meant to be instantiated directly'
+		raise NotImplementedError('This class is not meant to be instantiated directly')
 
 	def __iter__(self):
-		raise NotImplementedError
-
-	def list_names(self):
-		raise NotImplementedError
+		names = self.list_names()
+		return self._object_iter(names, True, True)
 
 	def list_files(self):
-		raise NotImplementedError
+		names = self.list_names()
+		return self._object_iter(names, True, False)
 
 	def list_folders(self):
+		names = self.list_names()
+		return self._object_iter(names, False, True)
+
+	def _object_iter(self, names, showfile, showdir):
+		raise NotImplementedError
+
+	def list_names(self, include_hidden=False):
 		raise NotImplementedError
 
 	def walk(self):
@@ -484,29 +458,33 @@ class Folder(FSObjectBase):
 	def child(self, path):
 		raise NotImplementedError
 
-	def new_file(self, path):
+	def new_file(self, path, check=None):
 		'''Get a L{File} object for a new file below this folder.
 		Like L{file()} but guarantees the file does not yet exist by
 		adding sequential numbers if needed. So the resulting file
 		may have a modified name.
 
 		@param path: the relative file path
+		@param check: a function that can check and reject the choice before it
+		is given back
 		@returns: a L{File} object
 		'''
-		return self._new_child(path, self.file)
+		return self._new_child(path, self.file, check)
 
-	def new_folder(self, path):
+	def new_folder(self, path, check=None):
 		'''Get a L{Folder} object for a new folder below this folder.
 		Like L{folder()} but guarantees the file does not yet exist by
 		adding sequential numbers if needed. So the resulting file
 		may have a modified name.
 
 		@param path: the relative file path
+		@param check: a function that can check and reject the choice before it
+		is given back
 		@returns: a L{Folder} object
 		'''
-		return self._new_child(path, self.folder)
+		return self._new_child(path, self.folder, check)
 
-	def _new_child(self, path, factory):
+	def _new_child(self, path, factory, check=None):
 		p = self.get_childpath(path.replace('%', '%%'))
 		if '.' in p.basename:
 			basename, ext = p.basename.split('.', 1)
@@ -520,13 +498,18 @@ class Folder(FSObjectBase):
 			try:
 				file = self.child(trypath) # this way we catch both exiting files and folders
 			except FileNotFoundError:
-				return factory(trypath)
+				child = factory(trypath)
+				if check is None or check(child):
+					return child
+				else:
+					logger.debug('File rejected by check "%s" trying increment', child.path)
 			else:
 				logger.debug('File exists "%s" trying increment', file.path)
-				i += 1
-				trypath = pattern % i
+
+			i += 1
+			trypath = pattern % i
 		else:
-			raise Exception, 'Could not find new file for: %s' % path
+			raise Exception('Could not find new file for: %s' % path)
 
 	def remove_children(self):
 		'''Recursively remove everything below this folder .
@@ -535,12 +518,12 @@ class Folder(FSObjectBase):
 		when executed for the wrong folder, so please make sure to double
 		check the dir is actually what you think it is before calling this.
 		'''
-		for child in self:
+		for name in self.list_names(include_hidden=True):
+			child = self.child(name)
 			assert child.path.startswith(self.path) # just to be real sure
 			if isinstance(child, Folder):
 				child.remove_children()
-			else:
-				child.remove()
+			child.remove()
 
 	def _copyto(self, other):
 		if other.exists():
@@ -568,7 +551,7 @@ except ImportError:
 
 #: Extensions to determine image mimetypes - used in L{File.isimage()}
 IMAGE_EXTENSIONS = (
-	# Gleaned from gtk.gdk.get_formats()
+	# Gleaned from Gdk.get_formats()
 	'bmp', # image/bmp
 	'gif', # image/gif
 	'icns', # image/x-icns
@@ -606,13 +589,12 @@ IMAGE_EXTENSIONS = (
 
 def _md5(content):
 	# Provide encoded content to avoid double work
-	if isinstance(content, basestring):
+	if isinstance(content, str):
 		content = (content,)
 
 	m = hashlib.md5()
 	for l in content:
-		l = l.encode('UTF-8') if isinstance(l, unicode) else l
-		m.update(l)
+		m.update(l.encode('UTF-8'))
 	return m.digest()
 
 
@@ -623,7 +605,7 @@ class File(FSObjectBase):
 	'''
 
 	def __init__(self, path, endofline=_EOL):
-		raise NotImplementedError, 'This class is not meant to be instantiated directly'
+		raise NotImplementedError('This class is not meant to be instantiated directly')
 
 	def __iter__(self):
 		return iter(self.readlines())
@@ -632,7 +614,7 @@ class File(FSObjectBase):
 		'''Check if this is an image file. Convenience method that
 		works even when no real mime-type suport is available.
 		If this method returns C{True} it is no guarantee
-		this image type is actually supported by gtk.
+		this image type is actually supported by Gtk.
 		@returns: C{True} when this is an image file
 		'''
 		# Quick shortcut to be able to load images in the gui even if
@@ -656,10 +638,14 @@ class File(FSObjectBase):
 				self._mimetype = str(mimetype)
 			else:
 				mimetype, encoding = mimetypes.guess_type(self.path, strict=False)
-				if encoding == 'gzip': return 'application/x-gzip'
-				elif encoding == 'bzip2': return 'application/x-bzip2'
-				elif encoding == 'compress': return 'application/x-compress'
-				else: self._mimetype = mimetype or 'application/octet-stream'
+				if encoding == 'gzip':
+					return 'application/x-gzip'
+				elif encoding == 'bzip2':
+					return 'application/x-bzip2'
+				elif encoding == 'compress':
+					return 'application/x-compress'
+				else:
+					self._mimetype = mimetype or 'application/octet-stream'
 
 		return self._mimetype
 
@@ -747,7 +733,7 @@ class File(FSObjectBase):
 			else:
 				return True
 		else:
-			raise AssertionError, 'Invalid etag: %r' % etag
+			raise AssertionError('Invalid etag: %r' % etag)
 
 	def _copyto(self, other):
 		if other.exists():
