@@ -1,8 +1,7 @@
-# -*- coding: utf-8 -*-
 
-# Copyright 2009-2017 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2009-2018 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+
 
-from __future__ import with_statement
 
 
 from datetime import datetime
@@ -46,16 +45,18 @@ class PagesIndexer(IndexerBase):
 
 	@signal: C{page-row-inserted (row)}: new row inserted
 	@signal: C{page-row-changed (row, oldrow)}: row changed
-	@signal: C{page-row-deleted (row)}: row to be deleted
+	@signal: C{page-row-delete (row)}: row to be deleted
+	@signal: C{page-row-deleted (row)}: row that has been deleted
 
 	@signal: C{page-changed (row, content)}: page contents changed
 	'''
 
 	__signals__ = {
 		'page-row-inserted': (None, None, (object,)),
-		'page-row-changed':  (None, None, (object, object)),
-		'page-row-deleted':  (None, None, (object,)),
-		'page-changed':      (None, None, (object, object))
+		'page-row-changed': (None, None, (object, object)),
+		'page-row-delete': (None, None, (object,)),
+		'page-row-deleted': (None, None, (object,)),
+		'page-changed': (None, None, (object, object))
 	}
 
 	def __init__(self, db, layout, filesindexer):
@@ -72,20 +73,25 @@ class PagesIndexer(IndexerBase):
 				n_children INTEGER DEFAULT 0,
 
 				name TEXT UNIQUE NOT NULL,
+				lowerbasename TEXT NOT NULL,
 				sortkey TEXT NOT NULL,
 				mtime TIMESTAMP,
 
 				source_file INTEGER REFERENCES files(id),
 				is_link_placeholder BOOLEAN DEFAULT 0
+
+				CONSTRAINT no_self_ref CHECK (parent <> id)
 			);
-			CREATE UNIQUE INDEX IF NOT EXISTS pages_name ON pages(name)
+			CREATE UNIQUE INDEX IF NOT EXISTS pages_name ON pages(name);
+			CREATE INDEX IF NOT EXISTS pages_sortkey ON pages(sortkey);
+			CREATE INDEX IF NOT EXISTS pages_parent ON pages(parent);
 		''')
 		row = self.db.execute('SELECT * FROM pages WHERE id == 1').fetchone()
 		if row is None:
 			c = self.db.execute(
-				'INSERT INTO pages(parent, name, sortkey, source_file) '
-				'VALUES (? , ?, ?, ?)',
-				(0, '', '', 1)
+				'INSERT INTO pages(parent, name, lowerbasename, sortkey, source_file) '
+				'VALUES (?, ?, ?, ?, ?)',
+				(0, '', '', '', 1)
 			)
 			assert c.lastrowid == 1 # ensure we start empty
 
@@ -174,7 +180,7 @@ class PagesIndexer(IndexerBase):
 		assert row is not None
 
 		if not row['is_link_placeholder']:
-			raise AssertionError, 'Not a placeholder'
+			raise AssertionError('Not a placeholder')
 		else:
 			self.remove_page(pagename)
 
@@ -188,19 +194,21 @@ class PagesIndexer(IndexerBase):
 			parent_row = self._select(pagename.parent)
 			assert parent_row is not None
 
-		# update table
+		# insert new page
+		lowerbasename = pagename.basename.lower()
 		sortkey = natural_sort_key(pagename.basename)
 		self.db.execute(
-			'INSERT INTO pages(name, sortkey, parent, is_link_placeholder, source_file)'
-			'VALUES (?, ?, ?, ?, ?)',
-			(pagename.name, sortkey, parent_row['id'], is_link_placeholder, file_id)
+			'INSERT INTO pages(name, lowerbasename, sortkey, parent, is_link_placeholder, source_file)'
+			'VALUES (?, ?, ?, ?, ?, ?)',
+			(pagename.name, lowerbasename, sortkey, parent_row['id'], is_link_placeholder, file_id)
 		)
-		self.update_parent(pagename.parent)
-
-		# notify others
 		row = self._select(pagename)
+		self._update_parent_nchildren(pagename.parent)
 		self.emit('page-row-inserted', row)
 
+		# update parent(s)
+		self.update_parent(pagename.parent)
+
 		return row['id']
 
 	def update_parent(self, parentname, allow_cleanup=lambda r: True, oldrow=None):
@@ -253,13 +261,33 @@ class PagesIndexer(IndexerBase):
 		# allow_cleanup is used by LinksIndexer when cleaning up placeholders
 
 		row = self._select(pagename)
+		assert row['id'] != 1, 'BUG: can\'t delete notebook root'
 		if row['n_children'] > 0:
-			raise AssertionError, 'Page has child pages'
+			raise AssertionError('Page has child pages')
 
-		self.emit('page-row-deleted', row)
+		self.emit('page-row-delete', row)
 		self.db.execute('DELETE FROM pages WHERE name=?', (pagename.name,))
+		self._update_parent_nchildren(pagename.parent)
+		self.emit('page-row-deleted', row)
 		self.update_parent(pagename.parent, allow_cleanup)
 
+	def _update_parent_nchildren(self, parentname):
+		# parent n_children needs to be up-to-date when we emit the "deleted"
+		# signal, else Gtk.TreeView sees an inconsistency
+		# We still call update_parent() after the fact to do the rest of the
+		# house keeping
+		row = self._select(parentname)
+		assert row is not None
+
+		n_children, = self.db.execute(
+			'SELECT count(*) FROM pages WHERE parent=?',
+			(row['id'],)
+		).fetchone()
+		self.db.execute(
+			'UPDATE pages SET n_children=? WHERE id=?',
+			(n_children, row['id'])
+		)
+
 
 class PageIndexRecord(Path):
 	'''Object representing a page L{Path} in the index, with data
@@ -309,7 +337,7 @@ class PagesViewInternal(object):
 			'SELECT * FROM pages WHERE id=?', (page_id,)
 		).fetchone()
 		if row is None:
-			raise IndexConsistencyError, 'No page for page_id "%r"' % page_id
+			raise IndexConsistencyError('No page for page_id "%r"' % page_id)
 		return PageIndexRecord(row)
 
 	def get_page_id(self, pagename):
@@ -317,26 +345,30 @@ class PagesViewInternal(object):
 			'SELECT id FROM pages WHERE name=?', (pagename.name,)
 		).fetchone()
 		if row is None:
-			raise IndexNotFoundError, 'Page not found in index: %s' % pagename.name
+			raise IndexNotFoundError('Page not found in index: %s' % pagename.name)
 		return row['id']
 
-	def resolve_link(self, source, href, ignore_link_placeholders=True):
-		if href.rel == HREF_REL_ABSOLUTE or source.isroot:
-			return self.resolve_pagename(ROOT_PATH, href.parts())
+	def resolve_link(self, source, href, ignore_link_placeholders=True, source_id=None):
+		parent, parent_id, names = self._resolve_link(source, href, ignore_link_placeholders, source_id)
+		return self.resolve_pagename(parent, names)
 
-		start, relnames = source, []
-		while True:
+	def _resolve_link(self, source, href, ignore_link_placeholders=True, source_id=None):
+		if href.rel == HREF_REL_ABSOLUTE or source.isroot:
+			return (ROOT_PATH, ROOT_ID, href.parts())
+
+		start, start_id, relnames = source, source_id, []
+		while start_id is None:
 			# Do not assume source exists, find start point that does
 			try:
 				start_id = self.get_page_id(start)
 			except IndexNotFoundError:
-				relnames.append(start.basename)
+				relnames.insert(0, start.basename)
 				start = start.parent
 			else:
 				break
 
 		if href.rel == HREF_REL_RELATIVE:
-			return self.resolve_pagename(start, relnames + href.parts())
+			return (start, start_id, relnames + href.parts())
 		else:
 			# HREF_REL_FLOATING
 			# Search upward namespaces for existing pages,
@@ -347,21 +379,21 @@ class PagesViewInternal(object):
 
 			if relnames:
 				# Check if we are anchored in non-existing part
-				keys = map(natural_sort_key, relnames)
+				keys = list(map(natural_sort_key, relnames))
 				if anchor_key in keys:
-					i = [c for c,k in enumerate(keys) if k==anchorkey][-1]
-					return self.resolve_pagename(db, root, relnames[:i] + href.parts()[1:])
+					i = [c for c, k in enumerate(keys) if k == anchor_key][-1]
+					return (start, start_id, relnames[:i] + href.parts())
 
 			if ignore_link_placeholders:
 				c = self.db.execute(
-					'SELECT name FROM pages '
+					'SELECT name, id FROM pages '
 					'WHERE sortkey=? and is_link_placeholder=0 '
 					'ORDER BY name DESC',
 					(anchor_key,)
 				) # sort longest first
 			else:
 				c = self.db.execute(
-					'SELECT name FROM pages '
+					'SELECT name, id FROM pages '
 					'WHERE sortkey=? '
 					'ORDER BY name DESC',
 					(anchor_key,)
@@ -370,7 +402,7 @@ class PagesViewInternal(object):
 			maxdepth = source.name.count(':')
 			depth = -1 # level where items were found
 			found = [] # candidates that match the link - these can only differ in case of the basename
-			for name, in c:
+			for name, pid in c:
 				mydepth = name.count(':')
 				if mydepth > maxdepth:
 					continue
@@ -381,39 +413,28 @@ class PagesViewInternal(object):
 					parentname = name.rsplit(':', 1)[0]
 					if start.name.startswith(parentname):
 						depth = mydepth
-						found.append(name)
+						found.append((name, pid))
 				else: # resolve from root namespace
-					found.append(name)
+					found.append((name, pid))
 
 			if found: # try to match case first, else just use first match
 				parts = href.parts()
 				anchor = parts.pop(0)
-				for name in found:
+				for name, pid in found:
 					if name.endswith(anchor):
-						return self.resolve_pagename(Path(name), parts)
+						return (Path(name), pid, parts)
 				else:
-					return self.resolve_pagename(Path(found[0]), parts)
+					name, pid = found[0]
+					return (Path(name), pid, parts)
 
 			else:
 				# Return "brother" of source
 				if relnames:
-					return self.resolve_pagename(start, relnames[:-1] + href.parts())
+					return (start, start_id, relnames[:-1] + href.parts())
 				else:
-					return self.resolve_pagename(start.parent, href.parts())
+					return (start.parent, None, href.parts())
 
-	def resolve_pagename(self, parent, names):
-		page_id, pagename, branch = self._resolve_pagename(parent, names)
-		if page_id is None:
-			try:
-				page_id = self.get_page_id(pagename)
-			except IndexNotFoundError:
-				pass
-			else:
-				logger.error('BUG: issue #19 seen: (%r, %r) -> (None, %r) - branch %i !', parent, names, pagename, branch)
-
-		return page_id, pagename
-
-	def _resolve_pagename(self, parent, names):
+	def resolve_pagename(self, parent, names, parent_id=None):
 		'''Resolve a pagename in the right case'''
 		# We do not ignore placeholders here. This can lead to a dependencies
 		# in how links are resolved based on order of indexing. However, this
@@ -421,37 +442,32 @@ class PagesViewInternal(object):
 		# if the tree for multiple links with slightly different spelling.
 		# Also we would need another call to return the page_id if a resolved
 		# page happens to exist.
+		assert isinstance(parent, Path)
 		pagename = parent
-		page_id = self.get_page_id(parent)
+		page_id = parent_id or self.get_page_id(parent)
 		for i, basename in enumerate(names):
-			if page_id == ROOT_ID:
-				row = self.db.execute(
-					'SELECT id, name FROM pages WHERE name=?',
-					(basename,)
-				).fetchone()
-			else:
-				row = self.db.execute(
-					'SELECT id, name FROM pages WHERE parent=? and name LIKE ?',
-					(page_id, "%:"+basename)
-				).fetchone()
+			sortkey = natural_sort_key(basename)
+			candidates = self.db.execute(
+				'SELECT id, name FROM pages '
+				'WHERE parent=? and sortkey=? ORDER BY name',
+				(page_id, sortkey)
+			).fetchall()
 
-			if row: # exact match
-				pagename = Path(row['name'])
-				page_id = row['id']
+			exact = pagename.child(basename).name
+			for row in candidates:
+				if row['name'] == exact:
+					pagename = Path(row['name'])
+					page_id = row['id']
+					break
 			else:
-				sortkey = natural_sort_key(basename)
-				row = self.db.execute(
-					'SELECT id, name FROM pages '
-					'WHERE parent=? and sortkey=? ORDER BY name',
-					(page_id, sortkey)
-				).fetchone()
-				if row: # case insensitive match
+				if candidates: # case insensitive match(es)
+					row = candidates[0]
 					pagename = Path(row['name'])
 					page_id = row['id']
 				else: # no match
-					return None, pagename.child(':'.join(names[i:])), 1
+					return None, pagename.child(':'.join(names[i:]))
 		else:
-			return page_id, pagename, 2
+			return page_id, pagename
 
 	def walk(self, parent_id):
 		# Need to do this recursive to preserve sorting
@@ -520,6 +536,37 @@ class PagesView(IndexView):
 		).fetchone()
 		return c
 
+	def match_pages(self, path, text, limit=10):
+		'''Generator for child pages of C{path} that match C{text} in their name
+		@param path: a L{Path} object
+		@param text: a string
+		@param limit: max number of results
+		@returns: yields L{Path} objects for children of C{path}
+		@raises IndexNotFoundError: if C{path} is not found in the index
+		'''
+		if path is None:
+			page_id = ROOT_ID
+		else:
+			page_id = self._pages.get_page_id(path) # can raise
+		return self._match_pages(page_id, text, limit)
+
+	def _match_pages(self, page_id, text, limit):
+		# The LIKE keyword does not handle unicode case-insensitivity
+		# therefore we need python lower() to do the job
+		for row in self.db.execute(
+			'SELECT * FROM pages WHERE parent=? and lowerbasename LIKE ? ORDER BY sortkey, name LIMIT ?',
+			(page_id, "%%%s%%" % text.lower(), limit)
+		):
+			yield PageIndexRecord(row)
+
+	def match_all_pages(self, text, limit=10):
+		'''Like C{match_pages()} except not limited a specific namespace'''
+		for row in self.db.execute(
+			'SELECT * FROM pages WHERE lowerbasename LIKE ? ORDER BY length(name), sortkey, name LIMIT ?',
+			("%%%s%%" % text.lower(), limit)
+		):
+			yield PageIndexRecord(row)
+
 	def walk(self, path=None):
 		'''Generator function to yield all pages in the index, depth
 		first
@@ -544,6 +591,26 @@ class PagesView(IndexView):
 		c, = self.db.execute('SELECT COUNT(*) FROM pages').fetchone()
 		return c - 1 # don't count ROOT
 
+	def get_has_previous_has_next(self, path):
+		if path.isroot:
+			raise ValueError('Can\'t use root')
+
+		r = self.db.execute(
+			'SELECT * FROM pages WHERE parent=? '
+			'ORDER BY sortkey ASC, name ASC LIMIT 1',
+			(ROOT_ID,)
+		).fetchone()
+		is_first = (r['name'] == path.name) if r else True
+
+		r = self.db.execute(
+			'SELECT * FROM pages WHERE parent=? '
+			'ORDER BY sortkey DESC, name DESC LIMIT 1',
+			(ROOT_ID,)
+		).fetchone()
+		is_last = (r['name'] == path.name) if r else True
+
+		return not is_first, not is_last
+
 	def get_previous(self, path):
 		'''Get the previous path in the index, in the same order that
 		L{walk()} will yield them
@@ -553,20 +620,23 @@ class PagesView(IndexView):
 		'''
 		# Find last (grand)child of previous item with same parent
 		# If no previous item, yield parent
-		if path.isroot: raise ValueError, 'Can\'t use root'
+		if path.isroot:
+			raise ValueError('Can\'t use root')
 
 		r = self.db.execute(
 			'SELECT parent FROM pages WHERE name=?', (path.name,)
 		).fetchone()
 		if r is None:
-			raise IndexNotFoundError, 'No such page: %s' % path
+			raise IndexNotFoundError('No such page: %s' % path)
 		else:
 			parent_id = r[0]
 
-		r = self.db.execute(
-			'SELECT * FROM pages WHERE parent=? and sortkey<? and name<? '
-			'ORDER BY sortkey DESC, name DESC LIMIT 1',
-			(parent_id, natural_sort_key(path.basename), path.name)
+		sortkey = natural_sort_key(path.basename)
+		r = self.db.execute('''
+			SELECT * FROM pages WHERE parent=? and (
+				sortkey<? or (sortkey=? and name<?)
+			) ORDER BY sortkey DESC, name DESC LIMIT 1''',
+			(parent_id, sortkey, sortkey, path.name)
 		).fetchone()
 		if not r:
 			parent = self._pages.get_pagename(parent_id)
@@ -579,7 +649,7 @@ class PagesView(IndexView):
 					(r['id'],)
 				).fetchone()
 				if r is None:
-					raise IndexConsistencyError, 'Missing children'
+					raise IndexConsistencyError('Missing children')
 			else:
 				return PageIndexRecord(r)
 
@@ -593,13 +663,14 @@ class PagesView(IndexView):
 		# If item has children, yield first child
 		# Else find next item with same parent
 		# If no next item, find next item for parent
-		if path.isroot: raise ValueError, 'Can\'t use root'
+		if path.isroot:
+			raise ValueError('Can\'t use root')
 
 		r = self.db.execute(
 			'SELECT * FROM pages WHERE name=?', (path.name,)
 		).fetchone()
 		if r is None:
-			raise IndexNotFoundError, 'No such page: %s' % path
+			raise IndexNotFoundError('No such page: %s' % path)
 
 		if r['n_children'] > 0:
 			r = self.db.execute(
@@ -608,15 +679,16 @@ class PagesView(IndexView):
 				(r['id'],)
 			).fetchone()
 			if r is None:
-				raise IndexConsistencyError, 'Missing children'
+				raise IndexConsistencyError('Missing children')
 			else:
 				return PageIndexRecord(r)
 		else:
 			while True:
-				n = self.db.execute(
-					'SELECT * FROM pages WHERE parent=? and sortkey>? and name>? '
-					'ORDER BY sortkey, name LIMIT 1',
-					(r['parent'], r['sortkey'], r['name'])
+				n = self.db.execute('''
+					SELECT * FROM pages WHERE parent=? and (
+						sortkey>? or (sortkey=? and name>?)
+					) ORDER BY sortkey, name LIMIT 1''',
+					(r['parent'], r['sortkey'], r['sortkey'], r['name'])
 				).fetchone()
 				if n is not None:
 					return PageIndexRecord(n)
@@ -627,7 +699,7 @@ class PagesView(IndexView):
 						'SELECT * FROM pages WHERE id=?', (r['parent'],)
 					).fetchone()
 					if r is None:
-						raise IndexConsistencyError, 'Missing parent'
+						raise IndexConsistencyError('Missing parent')
 
 	def lookup_from_user_input(self, name, reference=None):
 		'''Lookup a pagename based on user input
@@ -645,7 +717,7 @@ class PagesView(IndexView):
 		# Only accidental that we treat user input as links ... ;)
 		href = HRef.new_from_wiki_link(name)
 		if reference is None and href.rel == HREF_REL_RELATIVE:
-			raise ValueError, 'Got relative page name without parent: %s' % name
+			raise ValueError('Got relative page name without parent: %s' % name)
 		else:
 			source = reference or ROOT_PATH
 			id, pagename = self._pages.resolve_link(
@@ -729,6 +801,12 @@ class PagesView(IndexView):
 			yield PageIndexRecord(row)
 
 
+try:
+	from gi.repository import Gtk
+except ImportError:
+	Gtk = None
+
+
 IS_PAGE = 1 #: Hint for MyTreeIter
 
 class PagesTreeModelMixin(TreeModelMixinBase):
@@ -740,22 +818,45 @@ class PagesTreeModelMixin(TreeModelMixinBase):
 	# Signals use "find_all" instead of "find" to allow for subclasses that
 	# have multiple entries, like models for tags
 
+	def __init__(self, index, root=None, reverse=False):
+		TreeModelMixinBase.__init__(self, index)
+		self._REVERSE = reverse
+		if root is None:
+			self._MY_ROOT_NAME = ''
+			self._MY_ROOT_NAME_C = ''
+			self._MY_ROOT_ID = ROOT_ID
+		else:
+			self._MY_ROOT_NAME = root.name
+			self._MY_ROOT_NAME_C = root.name + ':'
+			self._set_root_id()
+
+		self._deleted_paths = None
+
+	def _set_root_id(self):
+		myrow = self.db.execute(
+			'SELECT * FROM pages WHERE name=?', (self._MY_ROOT_NAME,)
+		).fetchone()
+		self._MY_ROOT_ID = myrow['id'] if myrow else None
+
 	def connect_to_updateiter(self, index, update_iter):
 		self.connectto_all(update_iter.pages,
-			('page-row-inserted', 'page-row-changed', 'page-row-deleted')
+			('page-row-inserted', 'page-row-changed', 'page-row-delete', 'page-row-deleted')
 		)
 
 	def on_page_row_inserted(self, o, row):
 		self.flush_cache()
-		for treepath in self._find_all_pages(row['name']):
-			if treepath[-1] == 0 and len(treepath) > 1:
-				self._check_parent_has_child_toggled(treepath)
-			treeiter = self.get_iter(treepath) # not mytreeiter !
-			self.emit('row-inserted', treepath, treeiter)
+		if row['name'] == self._MY_ROOT_NAME:
+			self._set_root_id()
+		else:
+			for treepath in self._find_all_pages(row['name']):
+				treeiter = self.get_iter(treepath) # not mytreeiter !
+				self.emit('row-inserted', treepath, treeiter)
+				if treepath[-1] == 0 and len(treepath) > 1:
+					self._check_parent_has_child_toggled(treepath, 1)
 
-	def _check_parent_has_child_toggled(self, treepath):
+	def _check_parent_has_child_toggled(self, treepath, count):
 		parent = self.get_mytreeiter(treepath[:-1])
-		if parent.row['n_children'] == 1:
+		if parent.n_children == count:
 			treeiter = self.get_iter(parent.treepath) # not mytreeiter !
 			self.emit('row-has-child-toggled', parent.treepath, treeiter)
 
@@ -763,29 +864,48 @@ class PagesTreeModelMixin(TreeModelMixinBase):
 		# no clear cache here - just update row
 		for treepath in self._find_all_pages(row['name']):
 			treeiter = self.get_iter(treepath) # not mytreeiter !
-			self.cache[treepath].row = row # ensure uptodate info
+			self.cache[tuple(treepath)].row = row # ensure uptodate info
 			self.emit('row-changed', treepath, treeiter)
 
+	def on_page_row_delete(self, o, row):
+		self._deleted_paths = list(self._find_all_pages(row['name']))
+
 	def on_page_row_deleted(self, o, row):
+		# Technically "_deleted_paths" should always be a single path
+		# here, else two things changed at once, and Gtk.TreeView cannot
+		# always deal with that.
+
 		self.flush_cache()
-		for treepath in self._find_all_pages(row['name']):
-			if treepath[-1] == 0 and len(treepath) > 1:
-				self._check_parent_has_child_toggled(treepath)
-			self.emit('row-deleted', treepath)
+		if row['name'] == self._MY_ROOT_NAME:
+			self._MY_ROOT_ID = None
+		else:
+			for treepath in self._deleted_paths:
+				self.emit('row-deleted', treepath)
+				if treepath[-1] == 0 and len(treepath) > 1:
+					self._check_parent_has_child_toggled(treepath, 0)
+
+		self._deleted_paths = None
 
 	def n_children_top(self):
-		return self.db.execute(
-			'SELECT COUNT(*) FROM pages WHERE parent=?', (ROOT_ID,)
-		).fetchone()[0]
+		if self._MY_ROOT_ID is None:
+			return 0
+		else:
+			return self.db.execute(
+				'SELECT COUNT(*) FROM pages WHERE parent=?', (self._MY_ROOT_ID,)
+			).fetchone()[0]
 
 	def get_mytreeiter(self, treepath):
+		if self._MY_ROOT_ID is None:
+			return None
+
+		treepath = tuple(treepath) # used to cache
 		if treepath in self.cache:
 			return self.cache[treepath]
 
 		# Find parent
 		parentpath = treepath[:-1]
 		if not parentpath:
-			parent_id = ROOT_ID
+			parent_id = self._MY_ROOT_ID
 		else:
 			parent_iter = self.cache.get(parentpath, None) \
 							or self.get_mytreeiter(parentpath) # recurs
@@ -796,46 +916,71 @@ class PagesTreeModelMixin(TreeModelMixinBase):
 
 		# Now cache a slice at the target level
 		offset = treepath[-1]
-		for i, row in enumerate(self.db.execute('''
-			SELECT * FROM pages WHERE parent=?
-			ORDER BY sortkey, name LIMIT 20 OFFSET ?
-			''',
-			(parent_id, offset)
-		)):
-			mytreepath = parentpath + (offset + i,)
+		if self._REVERSE:
+			rows = self.db.execute('''
+				SELECT * FROM pages WHERE parent=?
+				ORDER BY sortkey DESC, name DESC LIMIT 20 OFFSET ?
+				''',
+				(parent_id, offset)
+			)
+		else:
+			rows = self.db.execute('''
+				SELECT * FROM pages WHERE parent=?
+				ORDER BY sortkey ASC, name ASC LIMIT 20 OFFSET ?
+				''',
+				(parent_id, offset)
+			)
+		for i, row in enumerate(rows):
+			mytreepath = tuple(parentpath) + (offset + i,)
 			if mytreepath not in self.cache:
-				self.cache[mytreepath] = MyTreeIter(mytreepath, row, row['n_children'], IS_PAGE)
+				self.cache[mytreepath] = MyTreeIter(
+					Gtk.TreePath(mytreepath),
+					row,
+					row['n_children'],
+					IS_PAGE
+				)
 			else:
 				break # avoid overwriting cache because of ref count
 
 		return self.cache.get(treepath, None)
 
 	def find(self, path):
+		'''Returns the C{Gtk.TreePath} for a notebook page L{Path}
+		If the L{Path} appears multiple times returns the first occurence
+		@raises IndexNotFoundError: if path not found
+		'''
 		if path.isroot:
 			raise ValueError
-		treepaths = self._find_all_pages(path.name)
-		treepaths.sort()
+		treepaths = sorted(self._find_all_pages(path.name))
 		try:
 			return treepaths[0]
 		except IndexError:
-			raise IndexNotFoundError, path
+			raise IndexNotFoundError(path)
 
 	def find_all(self, path):
+		'''Returns a list of C{Gtk.TreePath} for a notebook page L{Path}
+		Returns all occurences in the treeview
+		@raises IndexNotFoundError: if path not found
+		'''
 		if path.isroot:
 			raise ValueError
 		treepaths = self._find_all_pages(path.name)
 		if not treepaths:
-			raise IndexNotFoundError, path
+			raise IndexNotFoundError(path)
 		else:
 			return treepaths
 
 	def _find_all_pages(self, name, update_cache=True):
-		parent_id = ROOT_ID
-		names = name.split(':')
+		if self._MY_ROOT_ID is None or \
+			not name.startswith(self._MY_ROOT_NAME_C):
+				return []
+
+		parent_id = self._MY_ROOT_ID
+		names = name[len(self._MY_ROOT_NAME_C):].split(':')
 		treepath = []
 		for i, basename in enumerate(names):
 			# Get treepath
-			name = ':'.join(names[:i+1])
+			name = self._MY_ROOT_NAME_C + ':'.join(names[:i + 1])
 			myrow = self.db.execute(
 				'SELECT * FROM pages WHERE name=?', (name,)
 			).fetchone()
@@ -843,13 +988,22 @@ class PagesTreeModelMixin(TreeModelMixinBase):
 				raise IndexNotFoundError
 
 			sortkey = myrow['sortkey']
-			row = self.db.execute('''
-				SELECT COUNT(*) FROM pages
-				WHERE parent=? and (
-					sortkey<? or (sortkey=? and name<?)
-				)''',
-				(parent_id, sortkey, sortkey, name)
-			).fetchone()
+			if self._REVERSE:
+				row = self.db.execute('''
+					SELECT COUNT(*) FROM pages
+					WHERE parent=? and (
+						sortkey>? or (sortkey=? and name>?)
+					)''',
+					(parent_id, sortkey, sortkey, name)
+				).fetchone()
+			else:
+				row = self.db.execute('''
+					SELECT COUNT(*) FROM pages
+					WHERE parent=? and (
+						sortkey<? or (sortkey=? and name<?)
+					)''',
+					(parent_id, sortkey, sortkey, name)
+				).fetchone()
 			treepath.append(row[0])
 			parent_id = myrow['id']
 
@@ -857,10 +1011,15 @@ class PagesTreeModelMixin(TreeModelMixinBase):
 				# Update cache (avoid overwriting because of ref count)
 				mytreepath = tuple(treepath)
 				if mytreepath not in self.cache:
-					myiter = MyTreeIter(mytreepath, myrow, myrow['n_children'], IS_PAGE)
+					myiter = MyTreeIter(
+						Gtk.TreePath(mytreepath),
+						myrow,
+						myrow['n_children'],
+						IS_PAGE
+					)
 					self.cache[mytreepath] = myiter
 
-		return [tuple(treepath)]
+		return [Gtk.TreePath(treepath)]
 
 
 ########################################################################
