@@ -1,10 +1,10 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2008-2015 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 
 import re
 import logging
+import itertools
 
 logger = logging.getLogger('zim.notebook')
 
@@ -37,6 +37,42 @@ re.UNICODE)
 	# The UNICODE flag is used to make the alphanumber check international.
 
 
+def shortest_unique_names(paths):
+	'''Returns the shortest unique name for each path in paths
+	@param paths: list of L{Path} objects
+	@returns: list of strings
+	'''
+	by_basename = {}
+	for path in paths:
+		basename = path.basename
+		mylist = by_basename.setdefault(basename, [])
+		mylist.append(path)
+
+	result = []
+	for path in paths:
+		basename = path.basename
+		conflicts = by_basename[basename]
+		if len(conflicts) == 1:
+			result.append(path.basename)
+		else:
+			conflicts.remove(path)
+			conflicts.insert(0, path) # shuffle path of interest to front
+			reverse_paths = [reversed(p.name.split(':')) for p in conflicts]
+			names = []
+			for parts in itertools.zip_longest(*reverse_paths):
+				if parts[0] is None:
+					break
+				elif parts[0] not in parts[1:]:
+					names.append(parts[0])
+					break
+				else:
+					names.append(parts[0])
+
+			result.append(':'.join(reversed(names)))
+
+	return result
+
+
 class Path(object):
 	'''Class representing a page name in the notebook
 
@@ -97,11 +133,11 @@ class Path(object):
 		@param name: a string
 		@raises AssertionError: if the name is not valid
 		'''
-		assert isinstance(name, basestring)
+		assert isinstance(name, str)
 		if not name.strip(':') \
 		or _pagename_reduce_colon_re.search(name) \
 		or _pagename_invalid_char_re.search(name):
-			raise AssertionError, 'Not a valid page name: %s' % name
+			raise AssertionError('Not a valid page name: %s' % name)
 
 	@staticmethod
 	def makeValidPageName(name):
@@ -115,10 +151,11 @@ class Path(object):
 		'''
 		newname = _pagename_reduce_colon_re.sub(':', name.strip(':'))
 		newname = _pagename_invalid_char_re.sub('', newname)
+		newname = newname.replace('_', ' ')
 		try:
 			Path.assertValidPageName(newname)
 		except AssertionError:
-			raise ValueError, 'Not a valid page name: %s (was: %s)' % (newname, name)
+			raise ValueError('Not a valid page name: %s (was: %s)' % (newname, name))
 		return newname
 
 	def __init__(self, name):
@@ -141,16 +178,16 @@ class Path(object):
 			self.name = name.strip(':')
 
 		try:
-			self.name = unicode(self.name)
+			self.name = str(self.name)
 		except UnicodeDecodeError:
-			raise ValueError, 'BUG: invalid input, page names should be in ascii, or given as unicode'
+			raise ValueError('BUG: invalid input, page names should be in ascii, or given as unicode')
 
 	@classmethod
 	def new_from_zim_config(klass, string):
 		'''Returns a new object based on the string representation for
 		that path.
 		'''
-		return klass( klass.makeValidPageName(string) )
+		return klass(klass.makeValidPageName(string))
 
 	def serialize_zim_config(self):
 		'''Returns the name for serializing this path'''
@@ -219,10 +256,10 @@ class Path(object):
 		if path.name == '': # root path
 			return self.name
 		elif self.name.startswith(path.name + ':'):
-			i = len(path.name)+1
+			i = len(path.name) + 1
 			return self.name[i:].strip(':')
 		else:
-			raise ValueError, '"%s" is not below "%s"' % (self, path)
+			raise ValueError('"%s" is not below "%s"' % (self, path))
 
 	@property
 	def parent(self):
@@ -252,16 +289,22 @@ class Path(object):
 		@param basename: the relative name for the child
 		@returns: a new L{Path} object
 		'''
-		return Path(self.name+':'+basename)
+		return Path(self.name + ':' + basename)
 
 	def ischild(self, parent):
-		'''Check of this path is a child of a given path
-
+		'''Check whether this path is a child of a given path
 		@param parent: a L{Path} object
 		@returns: True when this path is a (grand-)child of C{parent}
 		'''
 		return parent.isroot or self.name.startswith(parent.name + ':')
 
+	def match_namespace(self, namespace):
+		'''Check whether this path is in a specific section of the notebook
+		@param namespace: a L{Path} object
+		@returns: True when this path is equal to C{namespace} or is a (grand-)child of C{namespace}
+		'''
+		return namespace.isroot or self.name == namespace.name or self.name.startswith(namespace.name + ':')
+
 	def commonparent(self, other):
 		'''Find a common parent for two Paths
 
@@ -343,10 +386,10 @@ class SourceFile(zim.fs.File):
 		return False
 
 	def write(self, *a):
-		raise AssertionError, 'Not writeable'
+		raise AssertionError('Not writeable')
 
 	def writelines(self, *a):
-		raise AssertionError, 'Not writeable'
+		raise AssertionError('Not writeable')
 
 
 class Page(Path, SignalEmitter):
@@ -395,7 +438,7 @@ class Page(Path, SignalEmitter):
 		self.modified = False
 		self._parsetree = None
 		self._ui_object = None
-
+		self._meta = None
 
 		self._readonly = None
 		self._last_etag = None
@@ -439,16 +482,33 @@ class Page(Path, SignalEmitter):
 
 	def _store_tree(self, tree):
 		if tree and tree.hascontent:
-			if not self.hascontent:
-				# New page
+			if self._meta is not None:
+				tree.meta.update(self._meta) # Preserver headers
+			elif self.source_file.exists():
+				# Try getting headers from file
+				try:
+					text = self.source_file.read()
+				except zim.newfs.FileNotFoundError:
+					return None
+				else:
+					parser = self.format.Parser()
+					tree = parser.parse(text)
+					self._meta = tree.meta
+					tree.meta.update(self._meta) # Preserver headers
+			else: # not self.source_file.exists()
 				now = datetime.now()
 				tree.meta['Creation-Date'] = now.isoformat()
 
-			lines = self.format.Dumper().dump(tree)
+			lines = self.format.Dumper().dump(tree, file_output=True)
 			self._last_etag = self.source_file.writelines_with_etag(lines, self._last_etag)
+			self._meta = tree.meta
 		else:
 			self.source_file.remove()
 			self._last_etag = None
+			self._meta = None
+
+	def check_source_changed(self):
+		self._check_source_etag()
 
 	def _check_source_etag(self):
 		if (
@@ -461,6 +521,7 @@ class Page(Path, SignalEmitter):
 		):
 			logger.info('Page changed on disk: %s', self.name)
 			self._last_etag = None
+			self._meta = None
 			self._parsetree = None
 			self.emit('page-changed', True)
 		else:
@@ -502,11 +563,13 @@ class Page(Path, SignalEmitter):
 		else:
 			try:
 				text, self._last_etag = self.source_file.read_with_etag()
-				parser = self.format.Parser()
-				self._parsetree = parser.parse(text)
 			except zim.newfs.FileNotFoundError:
 				return None
 			else:
+				parser = self.format.Parser()
+				self._parsetree = parser.parse(text, file_input=True)
+				self._meta = self._parsetree.meta
+				assert self._meta is not None
 				return self._parsetree
 
 	def set_parsetree(self, tree):
@@ -522,7 +585,7 @@ class Page(Path, SignalEmitter):
 		assert self.valid, 'BUG: page object became invalid'
 
 		if self.readonly:
-			raise PageReadOnlyError, self
+			raise PageReadOnlyError(self)
 
 		if self._ui_object:
 			self._ui_object.set_parsetree(tree)
@@ -574,7 +637,7 @@ class Page(Path, SignalEmitter):
 
 		@returns: text as a list of lines or an empty list
 		'''
-		if isinstance(format, basestring):
+		if isinstance(format, str):
 			format = zim.formats.get_format(format)
 
 		if not linker is None:
@@ -598,7 +661,7 @@ class Page(Path, SignalEmitter):
 		@param append: if C{True} the text is appended instead of
 		replacing current content.
 		'''
-		if isinstance(format, basestring):
+		if isinstance(format, str):
 			format = zim.formats.get_format(format)
 
 		if append:
