@@ -1,18 +1,19 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2011 NorfCran <norfcran@gmail.com>
 # License:  same as zim (gpl)
 
-from __future__ import with_statement
 
-import gtk
+
+from gi.repository import Gtk
 
 from zim.errors import Error
-from zim.plugins import PluginClass, extends, WindowExtension
+from zim.plugins import PluginClass
 from zim.actions import action
-from zim.gui.widgets import ui_environment, MessageDialog
 from zim.utils import natural_sort_key
 
+from zim.gui.pageview import PageViewExtension
+from zim.gui.widgets import MessageDialog
+
 
 import logging
 
@@ -36,28 +37,11 @@ If the list is already sorted the order will be reversed
 class NoSelectionError(Error):
 
 	def __init__(self):
-		Error.__init__(self,  _('Please select more than one line of text'))
+		Error.__init__(self, _('Please select more than one line of text'))
 			# T: Error message for linesorter plugin
 
 
-@extends('MainWindow')
-class MainWindowExtension(WindowExtension):
-
-	uimanager_xml = '''
-	<ui>
-		<menubar name='menubar'>
-			<menu action='edit_menu'>
-				<placeholder name='plugin_items'>
-					<menuitem action='remove_line'/>
-					<menuitem action='duplicate_line'/>
-					<menuitem action='move_line_up'/>
-					<menuitem action='move_line_down'/>
-					<menuitem action='sort_selected_lines'/>
-				</placeholder>
-			</menu>
-		</menubar>
-	</ui>
-	'''
+class LineSorterPageViewExtension(PageViewExtension):
 
 	def _get_selected_lines(self, buffer):
 		try:
@@ -83,9 +67,9 @@ class MainWindowExtension(WindowExtension):
 
 		return start, end
 
-	@action(_('_Sort lines'), stock='gtk-sort-ascending') # T: menu item
+	@action(_('_Sort lines'), menuhints='edit') # T: menu item
 	def sort_selected_lines(self):
-		buffer = self.window.pageview.view.get_buffer()
+		buffer = self.pageview.textview.get_buffer()
 		first_lineno, last_lineno = self._get_selected_lines(buffer)
 		if first_lineno == last_lineno:
 			raise NoSelectionError()
@@ -106,7 +90,7 @@ class MainWindowExtension(WindowExtension):
 			lines = []
 			for line_nr in range(first_lineno, last_lineno + 1):
 				start, end = buffer.get_line_bounds(line_nr)
-				text = buffer.get_text(start, end)
+				text = start.get_text(end)
 				tree = buffer.get_parsetree(bounds=(start, end))
 				lines.append((natural_sort_key(text), tree))
 			#~ logger.debug("Content of selected lines (text, tree): %s", lines)
@@ -125,7 +109,7 @@ class MainWindowExtension(WindowExtension):
 
 	def move_line(self, offset):
 		'''Move line at the current cursor position #offset lines down (up if offset is negative) '''
-		buffer = self.window.pageview.view.get_buffer()
+		buffer = self.pageview.textview.get_buffer()
 		start, end = self._get_iters_one_or_more_lines(buffer)
 
 		# do nothing if target is before begin or after end of document
@@ -169,32 +153,32 @@ class MainWindowExtension(WindowExtension):
 				buffer.place_cursor(iter)
 
 
-	@action(_('_Move Line Up'), accelerator='<Primary>Up', readonly=False)  # T: Menu item
+	@action(_('_Move Line Up'), accelerator='<Primary>Up', menuhints='edit')  # T: Menu item
 	def move_line_up(self):
 		'''Menu action to move line up'''
 		self.move_line(-1)
 
 
-	@action(_('_Move Line Down'), accelerator='<Primary>Down', readonly=False)  # T: Menu item
+	@action(_('_Move Line Down'), accelerator='<Primary>Down', menuhints='edit')  # T: Menu item
 	def move_line_down(self):
 		'''Menu action to move line down'''
 		self.move_line(1)
 
 
-	@action(_('_Duplicate Line'), accelerator='<Primary><Shift>D', readonly=False)  # T: Menu item
+	@action(_('_Duplicate Line'), accelerator='<Primary><Shift>D', menuhints='edit')  # T: Menu item
 	def duplicate_line(self):
 		'''Menu action to dublicate line'''
-		buffer = self.window.pageview.view.get_buffer()
+		buffer = self.pageview.textview.get_buffer()
 		start, end = self._get_iters_one_or_more_lines(buffer)
 		tree = buffer.get_parsetree(bounds=(start, end))
 		with buffer.user_action:
 			buffer.insert_parsetree(end, tree)
 
 
-	@action(_('_Remove Line'), accelerator='<Primary><Shift>K', readonly=False)  # T: Menu item
+	@action(_('_Remove Line'), accelerator='<Primary><Shift>K', menuhints='edit')  # T: Menu item
 	def remove_line(self):
 		'''Menu action to remove line at the current cursor position'''
-		buffer = self.window.pageview.view.get_buffer()
+		buffer = self.pageview.textview.get_buffer()
 		start, end = self._get_iters_one_or_more_lines(buffer)
 		buffer.delete(start, end)
 		buffer.set_modified(True)
