@@ -1,13 +1,15 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2009-2017 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
-import gtk
-import pango
+from gi.repository import Gtk
+from gi.repository import GObject
+from gi.repository import Pango
 
 import logging
 import re
 
+from zim.plugins import find_extension
+
 import zim.datetimetz as datetime
 from zim.utils import natural_sorted
 
@@ -18,27 +20,27 @@ from zim.gui.widgets import \
 	encode_markup_text, decode_markup_text
 from zim.gui.clipboard import Clipboard
 from zim.signals import DelayedCallback, SIGNAL_AFTER
+from zim.plugins import DialogExtensionBase, extendable
 
 logger = logging.getLogger('zim.plugins.tasklist')
 
-
 from .indexer import _MAX_DUE_DATE, _NO_TAGS, _date_re, _tag_re, _parse_task_labels, _task_labels_re
 
 
 class TaskListWidgetMixin(object):
 
 		def on_populate_popup(self, o, menu):
-			sep = gtk.SeparatorMenuItem()
+			sep = Gtk.SeparatorMenuItem()
 			menu.append(sep)
 
-			item = gtk.CheckMenuItem(_('Show Tasks as Flat List'))
+			item = Gtk.CheckMenuItem(_('Show Tasks as Flat List'))
 				# T: Checkbox in task list - hides parent items
 			item.set_active(self.uistate['show_flatlist'])
 			item.connect('toggled', self.on_show_flatlist_toggle)
 			item.show_all()
 			menu.append(item)
 
-			item = gtk.CheckMenuItem(_('Only Show Active Tasks'))
+			item = Gtk.CheckMenuItem(_('Only Show Active Tasks'))
 				# T: Checkbox in task list - this options hides tasks that are not yet started
 			item.set_active(self.uistate['only_show_act'])
 			item.connect('toggled', self.on_show_active_toggle)
@@ -56,69 +58,84 @@ class TaskListWidgetMixin(object):
 			self.task_list.set_flatlist(active)
 
 
-class TaskListWidget(gtk.VBox, TaskListWidgetMixin, WindowSidePaneWidget):
+class TaskListWidget(Gtk.VBox, TaskListWidgetMixin, WindowSidePaneWidget):
 
-	def __init__(self, tasksview, opener, preferences, uistate):
-		gtk.VBox.__init__(self)
+	title = _('Tasks') # T: tab label for side pane
+
+	def __init__(self, tasksview, opener, properties, uistate):
+		GObject.GObject.__init__(self)
 		self.uistate = uistate
 		self.uistate.setdefault('only_show_act', False)
 		self.uistate.setdefault('show_flatlist', False)
 
 		self.task_list = TaskListTreeView(
 			tasksview, opener,
-			_parse_task_labels(preferences['labels']),
-			nonactionable_tags=_parse_task_labels(preferences['nonactionable_tags']),
+			_parse_task_labels(properties['labels']),
+			nonactionable_tags=_parse_task_labels(properties['nonactionable_tags']),
 			filter_actionable=self.uistate['only_show_act'],
-			tag_by_page=preferences['tag_by_page'],
-			use_workweek=preferences['use_workweek'],
+			tag_by_page=properties['tag_by_page'],
+			use_workweek=properties['use_workweek'],
 			compact=True,
 			flatlist=self.uistate['show_flatlist'],
 		)
 		self.task_list.connect('populate-popup', self.on_populate_popup)
 		self.task_list.set_headers_visible(True)
 
+		self.connectto(properties, 'changed', self.on_properties_changed)
+
 		self.filter_entry = InputEntry(placeholder_text=_('Filter')) # T: label for filtering/searching tasks
 		self.filter_entry.set_icon_to_clear()
 		filter_cb = DelayedCallback(500,
 			lambda o: self.task_list.set_filter(self.filter_entry.get_text()))
 		self.filter_entry.connect('changed', filter_cb)
 
-		self.pack_start(ScrolledWindow(self.task_list))
-		self.pack_end(self.filter_entry, False)
+		self.pack_start(ScrolledWindow(self.task_list), True, True, 0)
+		self.pack_end(self.filter_entry, False, True, 0)
+
+	def on_properties_changed(self, properties):
+		self.task_list.update_properties(
+			task_labels=_parse_task_labels(properties['labels']),
+			nonactionable_tags=_parse_task_labels(properties['nonactionable_tags']),
+			tag_by_page=properties['tag_by_page'],
+			use_workweek=properties['use_workweek'],
+		)
 
 
+class TaskListDialogExtension(DialogExtensionBase):
+	pass
+
+@extendable(TaskListDialogExtension)
 class TaskListDialog(TaskListWidgetMixin, Dialog):
 
-	def __init__(self, window, tasksview, preferences):
-		Dialog.__init__(self, window, _('Task List'), # T: dialog title
-			buttons=gtk.BUTTONS_CLOSE, help=':Plugins:Task List',
-			defaultwindowsize=(550, 400) )
-		self.preferences = preferences
+	def __init__(self, parent, tasksview, properties):
+		Dialog.__init__(self, parent, _('Task List'), # T: dialog title
+			buttons=Gtk.ButtonsType.CLOSE, help=':Plugins:Task List',
+			defaultwindowsize=(550, 400))
+		self.properties = properties
 		self.tasksview = tasksview
+		self.notebook = parent.notebook
 
-		hbox = gtk.HBox(spacing=5)
-		self.vbox.pack_start(hbox, False)
+		hbox = Gtk.HBox(spacing=5)
+		self.vbox.pack_start(hbox, False, True, 0)
 		self.hpane = HPaned()
 		self.uistate.setdefault('hpane_pos', 75)
 		self.hpane.set_position(self.uistate['hpane_pos'])
-		self.vbox.add(self.hpane)
+		self.vbox.pack_start(self.hpane, True, True, 0)
 
 		# Task list
 		self.uistate.setdefault('only_show_act', False)
 		self.uistate.setdefault('show_flatlist', False)
 		self.uistate.setdefault('sort_column', 0)
-		self.uistate.setdefault('sort_order', int(gtk.SORT_DESCENDING))
+		self.uistate.setdefault('sort_order', int(Gtk.SortType.DESCENDING))
 
-		opener = window.get_resource_opener()
-		task_labels = _parse_task_labels(preferences['labels'])
-		nonactionable_tags = _parse_task_labels(preferences['nonactionable_tags'])
+		opener = parent.navigation
 		self.task_list = TaskListTreeView(
 			self.tasksview, opener,
-			task_labels,
-			nonactionable_tags=nonactionable_tags,
+			_parse_task_labels(properties['labels']),
+			nonactionable_tags=_parse_task_labels(properties['nonactionable_tags']),
 			filter_actionable=self.uistate['only_show_act'],
-			tag_by_page=preferences['tag_by_page'],
-			use_workweek=preferences['use_workweek'],
+			tag_by_page=properties['tag_by_page'],
+			use_workweek=properties['use_workweek'],
 			flatlist=self.uistate['show_flatlist'],
 			sort_column=self.uistate['sort_column'],
 			sort_order=self.uistate['sort_order']
@@ -128,14 +145,16 @@ class TaskListDialog(TaskListWidgetMixin, Dialog):
 		self.hpane.add2(ScrolledWindow(self.task_list))
 
 		# Tag list
-		self.tag_list = TagListTreeView(self.task_list, task_labels)
+		self.tag_list = TagListTreeView(self.task_list)
 		self.hpane.add1(ScrolledWindow(self.tag_list))
 
+		self.connectto(properties, 'changed', self.on_properties_changed)
+
 		# Filter input
-		hbox.pack_start(gtk.Label(_('Filter')+': '), False) # T: Input label
+		hbox.pack_start(Gtk.Label(_('Filter') + ': '), False, True, 0) # T: Input label
 		filter_entry = InputEntry()
 		filter_entry.set_icon_to_clear()
-		hbox.pack_start(filter_entry, False)
+		hbox.pack_start(filter_entry, False, True, 0)
 		filter_cb = DelayedCallback(500,
 			lambda o: self.task_list.set_filter(filter_entry.get_text()))
 		filter_entry.connect('changed', filter_cb)
@@ -148,16 +167,16 @@ class TaskListDialog(TaskListWidgetMixin, Dialog):
 				self.uistate['only_show_act'] = active
 				self.task_list.set_filter_actionable(active)
 
-		self.act_toggle = gtk.CheckButton(_('Only Show Active Tasks'))
+		self.act_toggle = Gtk.CheckButton.new_with_mnemonic(_('Only Show Active Tasks'))
 			# T: Checkbox in task list - this options hides tasks that are not yet started
 		self.act_toggle.set_active(self.uistate['only_show_act'])
 		self.act_toggle.connect('toggled', on_show_active_toggle)
 		self.uistate.connect('changed', lambda o: self.act_toggle.set_active(self.uistate['only_show_act']))
-		hbox.pack_start(self.act_toggle, False)
+		hbox.pack_start(self.act_toggle, False, True, 0)
 
 		# Statistics label
-		self.statistics_label = gtk.Label()
-		hbox.pack_end(self.statistics_label, False)
+		self.statistics_label = Gtk.Label()
+		hbox.pack_end(self.statistics_label, False, True, 0)
 
 		def set_statistics():
 			total = self.task_list.get_n_tasks()
@@ -177,15 +196,18 @@ class TaskListDialog(TaskListWidgetMixin, Dialog):
 			# make it less blocking - should be async preferably
 			# now it is at least on idle
 
-		### XXX HACK to get dependency to connect to
-		###   -- no access to plugin, so can;t use get_extension()
-		##    -- duplicat of this snippet in MainWindowExtension
-		for e in window.ui.notebook.__zim_extension_objects__:
-			if hasattr(e, 'indexer') and e.indexer.__class__.__name__ == 'TasksIndexer':
-				self.connectto(e, 'tasklist-changed', callback)
-				break
-		else:
-			raise AssertionError, 'Could not find tasklist notebook extension'
+		from . import TaskListNotebookExtension
+		nb_ext = find_extension(self.notebook, TaskListNotebookExtension)
+		self.connectto(nb_ext, 'tasklist-changed', callback)
+
+	def on_properties_changed(self, properties):
+		self.task_list.update_properties(
+			task_labels=_parse_task_labels(properties['labels']),
+			nonactionable_tags=_parse_task_labels(properties['nonactionable_tags']),
+			tag_by_page=properties['tag_by_page'],
+			use_workweek=properties['use_workweek'],
+		)
+		self.tag_list.refresh(self.task_list)
 
 	def do_response(self, response):
 		self.uistate['hpane_pos'] = self.hpane.get_position()
@@ -198,7 +220,7 @@ class TaskListDialog(TaskListWidgetMixin, Dialog):
 		else:
 			# if it is unsorted, just use the defaults
 			self.uistate['sort_column'] = TaskListTreeView.PRIO_COL
-			self.uistate['sort_order'] = gtk.SORT_ASCENDING
+			self.uistate['sort_order'] = Gtk.SortType.ASCENDING
 
 		Dialog.do_response(self, response)
 
@@ -214,19 +236,18 @@ class TagListTreeView(SingleClickTreeView):
 	_type_tag = 2
 	_type_untagged = 3
 
-	def __init__(self, task_list, task_labels):
-		model = gtk.ListStore(str, int, int, int) # tag name, number of tasks, type, weight
+	def __init__(self, task_list):
+		model = Gtk.ListStore(str, int, int, int) # tag name, number of tasks, type, weight
 		SingleClickTreeView.__init__(self, model)
-		self.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
+		self.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
 		self.task_list = task_list
-		self.task_labels = task_labels
 
-		column = gtk.TreeViewColumn(_('Tags'))
+		column = Gtk.TreeViewColumn(_('Tags'))
 			# T: Column header for tag list in Task List dialog
 		self.append_column(column)
 
-		cr1 = gtk.CellRendererText()
-		cr1.set_property('ellipsize', pango.ELLIPSIZE_END)
+		cr1 = Gtk.CellRendererText()
+		cr1.set_property('ellipsize', Pango.EllipsizeMode.END)
 		column.pack_start(cr1, True)
 		column.set_attributes(cr1, text=0, weight=3) # tag name, weight
 
@@ -246,7 +267,7 @@ class TagListTreeView(SingleClickTreeView):
 		tags = []
 		for row in self._get_selected():
 			if row[2] == self._type_tag:
-				tags.append(row[0].decode('utf-8'))
+				tags.append(row[0])
 			elif row[2] == self._type_untagged:
 				tags.append(_NO_TAGS)
 		return tags or None
@@ -256,14 +277,14 @@ class TagListTreeView(SingleClickTreeView):
 		labels = []
 		for row in self._get_selected():
 			if row[2] == self._type_label:
-				labels.append(row[0].decode('utf-8'))
+				labels.append(row[0])
 		return labels or None
 
 	def _get_selected(self):
 		selection = self.get_selection()
 		if selection:
 			model, paths = selection.get_selected_rows()
-			if not paths or (0,) in paths:
+			if not paths or any(p == Gtk.TreePath(0) for p in paths):
 				return []
 			else:
 				return [model[path] for path in paths]
@@ -276,27 +297,28 @@ class TagListTreeView(SingleClickTreeView):
 
 		# Rebuild model
 		model = self.get_model()
-		if model is None: return
+		if model is None:
+				return
 		model.clear()
 
 		n_all = self.task_list.get_n_tasks()
-		model.append((_('All Tasks'), n_all, self._type_label, pango.WEIGHT_BOLD)) # T: "tag" for showing all tasks
+		model.append((_('All Tasks'), n_all, self._type_label, Pango.Weight.BOLD)) # T: "tag" for showing all tasks
 
 		used_labels = self.task_list.get_labels()
-		for label in self.task_labels: # explicitly keep sorting from preferences
+		for label in self.task_list.task_labels: # explicitly keep sorting from properties
 			if label in used_labels:
-				model.append((label, used_labels[label], self._type_label, pango.WEIGHT_BOLD))
+				model.append((label, used_labels[label], self._type_label, Pango.Weight.BOLD))
 
 		tags = self.task_list.get_tags()
 		if _NO_TAGS in tags:
 			n_untagged = tags.pop(_NO_TAGS)
-			model.append((_('Untagged'), n_untagged, self._type_untagged, pango.WEIGHT_NORMAL))
+			model.append((_('Untagged'), n_untagged, self._type_untagged, Pango.Weight.NORMAL))
 			# T: label in tasklist plugins for tasks without a tag
 
 		model.append(('', 0, self._type_separator, 0)) # separator
 
 		for tag in natural_sorted(tags):
-			model.append((tag, tags[tag], self._type_tag, pango.WEIGHT_NORMAL))
+			model.append((tag, tags[tag], self._type_tag, Pango.Weight.NORMAL))
 
 		# Restore selection
 		def reselect(model, path, iter):
@@ -324,10 +346,14 @@ ALERT_COLOR = '#FCEB65' # yellow ("idem" - #FCE94F)
 COLORS = [None, ALERT_COLOR, MEDIUM_COLOR, HIGH_COLOR] # index 0..3
 
 def days_to_str(days):
-	if days > 290:  return '%iy' % round(float(days)/365) # round up to 1 year from ~10 months
-	elif days > 25: return '%im' % round(float(days)/30)
-	elif days > 10: return '%iw' % round(float(days)/7)
-	else:           return '%id' % days
+	if days > 290:
+			return '%iy' % round(float(days) / 365) # round up to 1 year from ~10 months
+	elif days > 25:
+			return '%im' % round(float(days) / 30)
+	elif days > 10:
+			return '%iw' % round(float(days) / 7)
+	else:
+			return '%id' % days
 
 
 class TaskListTreeView(BrowserTreeView):
@@ -356,13 +382,13 @@ class TaskListTreeView(BrowserTreeView):
 		nonactionable_tags=(),
 		filter_actionable=False, tag_by_page=False, use_workweek=False,
 		compact=False, flatlist=False,
-		sort_column=PRIO_COL, sort_order=gtk.SORT_DESCENDING
+		sort_column=PRIO_COL, sort_order=Gtk.SortType.DESCENDING
 	):
-		self.real_model = gtk.TreeStore(bool, bool, int, str, str, object, str, str, int, int, str)
+		self.real_model = Gtk.TreeStore(bool, bool, int, str, str, object, str, str, int, int, str)
 			# VIS_COL, ACT_COL, PRIO_COL, START_COL, DUE_COL, TAGS_COL, DESC_COL, PAGE_COL, TASKID_COL, PRIO_SORT_COL, PRIO_SORT_LABEL_COL
 		model = self.real_model.filter_new()
 		model.set_visible_column(self.VIS_COL)
-		model = gtk.TreeModelSort(model)
+		model = Gtk.TreeModelSort(model)
 		model.set_sort_column_id(sort_column, sort_order)
 		BrowserTreeView.__init__(self, model)
 
@@ -380,7 +406,7 @@ class TaskListTreeView(BrowserTreeView):
 		self.flatlist = flatlist
 
 		# Add some rendering for the Prio column
-		def render_prio(col, cell, model, i):
+		def render_prio(col, cell, model, i, data):
 			prio = model.get_value(i, self.PRIO_COL)
 			text = model.get_value(i, self.PRIO_SORT_LABEL_COL)
 			if text.startswith('>'):
@@ -391,16 +417,16 @@ class TaskListTreeView(BrowserTreeView):
 			cell.set_property('markup', text)
 			cell.set_property('cell-background', bg)
 
-		cell_renderer = gtk.CellRendererText()
-		column = gtk.TreeViewColumn('!', cell_renderer)
+		cell_renderer = Gtk.CellRendererText()
+		column = Gtk.TreeViewColumn('!', cell_renderer)
 		column.set_cell_data_func(cell_renderer, render_prio)
 		column.set_sort_column_id(self.PRIO_SORT_COL)
 		self.append_column(column)
 
 		# Rendering for task description column
-		cell_renderer = gtk.CellRendererText()
-		cell_renderer.set_property('ellipsize', pango.ELLIPSIZE_END)
-		column = gtk.TreeViewColumn(_('Task'), cell_renderer, markup=self.DESC_COL)
+		cell_renderer = Gtk.CellRendererText()
+		cell_renderer.set_property('ellipsize', Pango.EllipsizeMode.END)
+		column = Gtk.TreeViewColumn(_('Task'), cell_renderer, markup=self.DESC_COL)
 				# T: Column header Task List dialog
 		column.set_resizable(True)
 		column.set_sort_column_id(self.DESC_COL)
@@ -412,11 +438,9 @@ class TaskListTreeView(BrowserTreeView):
 		self.append_column(column)
 		self.set_expander_column(column)
 
-		if gtk.gtk_version >= (2, 12) \
-		and gtk.pygtk_version >= (2, 12):
-			# custom tooltip
-			self.props.has_tooltip = True
-			self.connect("query-tooltip", self._query_tooltip_cb)
+		# custom tooltip
+		self.props.has_tooltip = True
+		self.connect("query-tooltip", self._query_tooltip_cb)
 
 		# Rendering of the Date column
 		day_of_week = datetime.date.today().isoweekday()
@@ -429,10 +453,10 @@ class TaskListTreeView(BrowserTreeView):
 		else:
 			delta1, delta2 = 1, 2
 
-		today    = str( datetime.date.today() )
-		tomorrow = str( datetime.date.today() + datetime.timedelta(days=delta1))
-		dayafter = str( datetime.date.today() + datetime.timedelta(days=delta2))
-		def render_date(col, cell, model, i):
+		today = str(datetime.date.today())
+		tomorrow = str(datetime.date.today() + datetime.timedelta(days=delta1))
+		dayafter = str(datetime.date.today() + datetime.timedelta(days=delta2))
+		def render_date(col, cell, model, i, data):
 			date = model.get_value(i, self.DUE_COL)
 			if date == _MAX_DUE_DATE:
 				cell.set_property('text', '')
@@ -440,16 +464,20 @@ class TaskListTreeView(BrowserTreeView):
 				cell.set_property('text', date)
 				# TODO allow strftime here
 
-			if date <= today: color = HIGH_COLOR
-			elif date <= tomorrow: color = MEDIUM_COLOR
-			elif date <= dayafter: color = ALERT_COLOR
+			if date <= today:
+					color = HIGH_COLOR
+			elif date <= tomorrow:
+					color = MEDIUM_COLOR
+			elif date <= dayafter:
+					color = ALERT_COLOR
 				# "<=" because tomorrow and/or dayafter can be after the weekend
-			else: color = None
+			else:
+					color = None
 			cell.set_property('cell-background', color)
 
 		if not compact:
-			cell_renderer = gtk.CellRendererText()
-			column = gtk.TreeViewColumn(_('Date'), cell_renderer)
+			cell_renderer = Gtk.CellRendererText()
+			column = Gtk.TreeViewColumn(_('Date'), cell_renderer)
 				# T: Column header Task List dialog
 			column.set_cell_data_func(cell_renderer, render_date)
 			column.set_sort_column_id(self.DUE_COL)
@@ -457,8 +485,8 @@ class TaskListTreeView(BrowserTreeView):
 
 		# Rendering for page name column
 		if not compact:
-			cell_renderer = gtk.CellRendererText()
-			column = gtk.TreeViewColumn(_('Page'), cell_renderer, text=self.PAGE_COL)
+			cell_renderer = Gtk.CellRendererText()
+			column = Gtk.TreeViewColumn(_('Page'), cell_renderer, text=self.PAGE_COL)
 					# T: Column header Task List dialog
 			column.set_sort_column_id(self.PAGE_COL)
 			self.append_column(column)
@@ -470,6 +498,26 @@ class TaskListTreeView(BrowserTreeView):
 		self.connect('row_activated', self.__class__.do_row_activated)
 		self.connect('focus-in-event', self.__class__.do_focus_in_event)
 
+	def update_properties(self,
+		task_labels=None,
+		nonactionable_tags=None,
+		tag_by_page=None,
+		use_workweek=None,
+	):
+		if task_labels is not None:
+			self.task_labels = task_labels
+
+		if nonactionable_tags is not None:
+			self.nonactionable_tags = tuple(t.strip('@').lower() for t in nonactionable_tags)
+
+		if tag_by_page is not None:
+			self.tag_by_page = tag_by_page
+
+		if use_workweek is not None:
+			print("TODO udate_use_workweek rendering")
+
+		self.refresh()
+
 	def refresh(self):
 		'''Refresh the model based on index data'''
 		# Update data
@@ -486,7 +534,8 @@ class TaskListTreeView(BrowserTreeView):
 			if tag[0] == prev[0]:
 				self._tags[prev[1]] += self._tags[tag[1]]
 				self._tags.pop(tag[1])
-			prev = tag
+			else:
+				prev = tag
 
 		# Set view
 		self._eval_filter() # keep current selection
@@ -543,7 +592,7 @@ class TaskListTreeView(BrowserTreeView):
 				actionable = False
 				y, m, d = row['start'].split('-')
 				td = datetime.date(int(y), int(m), int(d)) - today
-				prio_sort_label = '>'+ days_to_str(td.days)
+				prio_sort_label = '>' + days_to_str(td.days)
 				if row['prio'] > 0:
 					prio_sort_label += ' ' + '!' * min(row['prio'], 3)
 			elif row['due'] < _MAX_DUE_DATE:
@@ -551,9 +600,12 @@ class TaskListTreeView(BrowserTreeView):
 				td = datetime.date(int(y), int(m), int(d)) - today
 				prio_sort_label = \
 					'!' * min(row['prio'], 3) + ' ' if row['prio'] > 0 else ''
-				if td.days < 0:     prio_sort_label += '<b><u>OD</u></b>' # over due
-				elif td.days == 0:  prio_sort_label += '<u>TD</u>' # today
-				else:               prio_sort_label += days_to_str(td.days)
+				if td.days < 0:
+						prio_sort_label += '<b><u>OD</u></b>' # over due
+				elif td.days == 0:
+						prio_sort_label += '<u>TD</u>' # today
+				else:
+						prio_sort_label += days_to_str(td.days)
 			else:
 				prio_sort_label = '!' * min(row['prio'], 3)
 
@@ -660,8 +712,8 @@ class TaskListTreeView(BrowserTreeView):
 		if not modelrow[self.ACT_COL] and self.filter_actionable:
 			visible = False
 
-		description = modelrow[self.DESC_COL].decode('utf-8').lower()
-		pagename = modelrow[self.PAGE_COL].decode('utf-8').lower()
+		description = modelrow[self.DESC_COL].lower()
+		pagename = modelrow[self.PAGE_COL].lower()
 		tags = [t.lower() for t in modelrow[self.TAGS_COL]]
 
 		if visible and self.label_filter:
@@ -700,7 +752,7 @@ class TaskListTreeView(BrowserTreeView):
 
 	def do_row_activated(self, path, column):
 		model = self.get_model()
-		page = Path( model[path][self.PAGE_COL] )
+		page = Path(model[path][self.PAGE_COL])
 		text = self._get_raw_text(model[path])
 
 		pageview = self.opener.open_page(page)
@@ -712,7 +764,7 @@ class TaskListTreeView(BrowserTreeView):
 		return row['description']
 
 	def do_initialize_popup(self, menu):
-		item = gtk.ImageMenuItem('gtk-copy')
+		item = Gtk.MenuItem.new_with_mnemonic(_('_Copy')) # T: menu label
 		item.connect('activate', self.copy_to_clipboard)
 		menu.append(item)
 		self.populate_popup_expand_collapse(menu)
@@ -723,13 +775,16 @@ class TaskListTreeView(BrowserTreeView):
 		if not context:
 			return False
 
-		model, path, iter = context
+		model, iter = context.model, context.iter
+		if not (model and iter):
+			return
+
 		task = model[iter][self.DESC_COL]
 		start = model[iter][self.START_COL]
 		due = model[iter][self.DUE_COL]
 		page = model[iter][self.PAGE_COL]
 
-		today    = str( datetime.date.today() )
+		today = str(datetime.date.today())
 
 		text = [task, '\n']
 		if start and start > today:
@@ -800,19 +855,27 @@ class TaskListTreeView(BrowserTreeView):
 <tr><th>Prio</th><th>Task</th><th>Date</th><th>Page</th></tr>
 ''' % (HIGH_COLOR, MEDIUM_COLOR, ALERT_COLOR)
 
-		today    = str( datetime.date.today() )
-		tomorrow = str( datetime.date.today() + datetime.timedelta(days=1))
-		dayafter = str( datetime.date.today() + datetime.timedelta(days=2))
+		today = str(datetime.date.today())
+		tomorrow = str(datetime.date.today() + datetime.timedelta(days=1))
+		dayafter = str(datetime.date.today() + datetime.timedelta(days=2))
 		for indent, prio, desc, date, page in self.get_visible_data():
-			if prio >= 3: prio = '<td class="high">%s</td>' % prio
-			elif prio == 2: prio = '<td class="medium">%s</td>' % prio
-			elif prio == 1: prio = '<td class="alert">%s</td>' % prio
-			else: prio = '<td>%s</td>' % prio
+			if prio >= 3:
+					prio = '<td class="high">%s</td>' % prio
+			elif prio == 2:
+					prio = '<td class="medium">%s</td>' % prio
+			elif prio == 1:
+					prio = '<td class="alert">%s</td>' % prio
+			else:
+					prio = '<td>%s</td>' % prio
 
-			if date and date <= today: date = '<td class="high">%s</td>' % date
-			elif date == tomorrow: date = '<td class="medium">%s</td>' % date
-			elif date == dayafter: date = '<td class="alert">%s</td>' % date
-			else: date = '<td>%s</td>' % date
+			if date and date <= today:
+					date = '<td class="high">%s</td>' % date
+			elif date == tomorrow:
+					date = '<td class="medium">%s</td>' % date
+			elif date == dayafter:
+					date = '<td class="alert">%s</td>' % date
+			else:
+					date = '<td>%s</td>' % date
 
 			desc = '<td>%s%s</td>' % ('&nbsp;' * (4 * indent), desc)
 			page = '<td>%s</td>' % page
@@ -836,9 +899,9 @@ class TaskListTreeView(BrowserTreeView):
 
 			row = model[iter]
 			prio = row[self.PRIO_COL]
-			desc = row[self.DESC_COL].decode('utf-8')
+			desc = row[self.DESC_COL]
 			date = row[self.DUE_COL]
-			page = row[self.PAGE_COL].decode('utf-8')
+			page = row[self.PAGE_COL]
 
 			if date == _MAX_DUE_DATE:
 				date = ''
@@ -849,7 +912,3 @@ class TaskListTreeView(BrowserTreeView):
 		model.foreach(collect)
 
 		return rows
-
-# Need to register classes defining gobject signals
-#~ gobject.type_register(TaskListTreeView)
-# NOTE: enabling this line causes this treeview to have wrong theming under default ubuntu them !???
