@@ -1,9 +1,8 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2009-2012 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 # Copyright 2012 Damien Accorsi <damien.accorsi@free.fr>
 
-from __future__ import with_statement
+
 
 import os
 import logging
@@ -12,6 +11,7 @@ import xml.etree.ElementTree # needed to compile with cElementTree
 import xml.etree.cElementTree as ET
 
 
+from zim.newfs import LocalFile
 from zim.plugins.versioncontrol import VCSApplicationBase
 from zim.applications import Application
 
@@ -21,11 +21,9 @@ logger = logging.getLogger('zim.vcs.hg')
 
 class HGApplicationBackend(VCSApplicationBase):
 
-	use_staging = False
-
 	@classmethod
 	def build_bin_application_instance(cls):
-		return Application(('hg', '--noninteractive', '--encoding', 'utf8'), encoding='utf-8')
+		return Application(('hg', '--noninteractive'))
 			# force hg to run in non-interactive mode
 			# which will force user name to be auto-setup
 
@@ -49,8 +47,7 @@ class HGApplicationBackend(VCSApplicationBase):
 		if isinstance(versions, (tuple, list)):
 			assert 1 <= len(versions) <= 2
 			if len(versions) == 2:
-				versions = map(int, versions)
-				versions.sort()
+				versions = sorted(map(int, versions))
 				return ['-r', '%i..%i' % tuple(versions)]
 			else:
 				versions = versions[0]
@@ -75,7 +72,7 @@ class HGApplicationBackend(VCSApplicationBase):
 			return self.run(['add', path])
 
 
-	def annotate(self, file, version):
+	def annotate(self, file, version=None):
 		"""FIXME Document
 		return
 		0: line1
@@ -97,14 +94,14 @@ class HGApplicationBackend(VCSApplicationBase):
 		Runs: hg commit -m {{MSG}} {{PATH}}
 		"""
 		params = ['commit']
-		if msg!='' and msg!=None:
+		if msg != '' and msg is not None:
 			params.append('-m')
 			params.append(msg)
-		if path!='' and path!=None:
+		if path != '' and path is not None:
 			params.append(path)
 		return self.run(params)
 
-	def diff(self, versions, path=None):
+	def diff(self, versions=None, path=None):
 		"""
 		Runs:
 			hg diff --git {{REVISION_ARGS}}
@@ -112,7 +109,7 @@ class HGApplicationBackend(VCSApplicationBase):
 			hg diff --git {{REVISION_ARGS}} {{PATH}}
 		"""
 		revision_args = self.build_revision_arguments(versions)
-		if path==None:
+		if path is None:
 			return self.pipe(['diff', '--git'] + revision_args)
 			# Using --git option allow to show the renaming of files
 		else:
@@ -127,7 +124,7 @@ class HGApplicationBackend(VCSApplicationBase):
 		@returns: nothing
 		"""
 		#TODO: append the rule instead of overwrite the full content
-		self.root.file( '.hgignore' ).write( file_to_ignore_regexp )
+		self.root.file('.hgignore').write(file_to_ignore_regexp)
 
 
 	def init_repo(self):
@@ -147,7 +144,7 @@ class HGApplicationBackend(VCSApplicationBase):
 
 		@returns: a boolean True if a repo is already setup, or False
 		"""
-		return self.root.subdir('.hg').exists()
+		return self.root.folder('.hg').exists()
 
 	def init(self):
 		"""
@@ -160,7 +157,7 @@ class HGApplicationBackend(VCSApplicationBase):
 		@returns: True if the repo is not up-to-date, or False
 		"""
 		# If status return an empty answer, this means the local repo is up-to-date
-		return ''.join( self.status() ).strip() != ''
+		return ''.join(self.status()).strip() != ''
 
 	def log(self, path=None):
 		"""
@@ -176,7 +173,7 @@ class HGApplicationBackend(VCSApplicationBase):
 		versions = []
 		xml = ET.fromstring(''.join(log_op_output))
 		if not (xml and xml.tag == 'log'):
-			raise AssertionError, 'Could not parse log'
+			raise AssertionError('Could not parse log')
 		for entry in xml:
 			rev = entry.attrib['revision']
 			date = entry.findtext('date')
@@ -197,9 +194,10 @@ class HGApplicationBackend(VCSApplicationBase):
 		"""
 		Runs: hg rm {{PATH}}
 		"""
-		return self.run(['rm', '-A', path])
+		if isinstance(path, LocalFile):
+			return self.run(['rm', '-A', path])
 
-	def revert(self, path, version):
+	def revert(self, path=None, version=None):
 		"""
 		Runs:
 			hg revert --no-backup {{PATH}} {{REV_ARGS}}
