@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2009 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
@@ -142,11 +141,15 @@ class Query(object):
 				tokens.append(operators[w.lower()])
 			elif keyword_re.match(w):
 				keyword = keyword_re[1].lower()
-				string = keyword_re[2] or words.pop(0)
-				string = unescape_quoted_string(string)
-				if keyword == 'links':
-					keyword = 'linksfrom'
-				tokens.append(QueryTerm(keyword, string))
+				if not (keyword_re[2] or words):
+					# edge case - something ending in ":" but nothing following
+					tokens.append(QueryTerm('contentorname', keyword_re[1]+":")) # default keyword
+				else:
+					string = keyword_re[2] or words.pop(0)
+					string = unescape_quoted_string(string)
+					if keyword == 'links':
+						keyword = 'linksfrom'
+					tokens.append(QueryTerm(keyword, string))
 			else:
 				w = unescape_quoted_string(w)
 				if tag_re.match(w):
@@ -404,7 +407,7 @@ class SearchSelection(PageSelection):
 			else:
 				regex = self._name_regex(term.string)
 
-			#~ print '!! REGEX: ' + regex.pattern
+			#~ print('!! REGEX: ' + regex.pattern)
 			for path in generator:
 				if regex.match(path.name):
 					myresults.add(path)
@@ -501,13 +504,13 @@ class SearchSelection(PageSelection):
 						pass
 			generator = page_generator()
 		else:
-			generator = map(self.notebook.get_page, self.notebook.pages.walk())
+			generator = list(map(self.notebook.get_page, self.notebook.pages.walk()))
 
 		if results is None:
 			results = SearchSelection(None)
 
 		for page in generator:
-			#~ print '!! Search content', page
+			#~ print('!! Search content', page)
 			try:
 				tree = page.get_parsetree()
 			except:
@@ -521,7 +524,7 @@ class SearchSelection(PageSelection):
 			if operator == OPERATOR_AND:
 				score = 0
 				for term in terms:
-					#~ print '!! Count AND %s' % term
+					#~ print('!! Count AND %s' % term)
 					myscore = tree.countre(term.content_regex)
 					if term.keyword == 'contentorname' \
 					and term.name_regex.match(path.name):
@@ -538,7 +541,7 @@ class SearchSelection(PageSelection):
 					self._count_score(path, score)
 			else: # OPERATOR_OR
 				for term in terms:
-					#~ print '!! Count OR %s' % term
+					#~ print('!! Count OR %s' % term)
 					score = tree.countre(term.content_regex)
 					if term.keyword == 'contentorname' \
 					and term.name_regex.match(path.name):
@@ -596,8 +599,6 @@ class SearchSelection(PageSelection):
 		# Build a regex for a content search term, expands wildcards
 		# and sets case sensitivity. Tries to guess if we look for
 		# whole word or not.
-		if not isinstance(string, unicode):
-			string = string.decode('UTF-8')
 
 		# Build regex - first expand wildcards
 		parts = string.split('*')
@@ -608,14 +609,14 @@ class SearchSelection(PageSelection):
 		# charaters. Chinese is treated special because it does not use
 		# whitespace as word delimiter.
 		if re.search(r'^[\*\w]', string, re.U) \
-		and not u'\u4e00' <= string[0] <= u'\u9fff':
+		and not '\u4e00' <= string[0] <= '\u9fff':
 			regex = r'\b' + regex
 
 		if re.search(r'[\*\w]$', string, re.U) \
-		and not u'\u4e00' <= string[-1] <= u'\u9fff':
+		and not '\u4e00' <= string[-1] <= '\u9fff':
 			regex = regex + r'\b'
 
-		#~ print 'SEARCH REGEX: >>%s<<' % regex
+		#~ print('SEARCH REGEX: >>%s<<' % regex)
 		if case:
 			return re.compile(regex, re.U)
 		else:
